      SUBROUTINE genpdp(lda,n,ia,ilow,ihgh,type,
     &                  iidx,iatmp,a,coef)

      IMPLICIT none

      include 'genpdp.h'

!  .. Scalar variables ..

      INTEGER lda, n, type

!  .. Array Arguments ..

      INTEGER ia(lda,*)
      INTEGER ilow(*), ihgh(*)

!  .. PURPOSE ..
  
!     For an n by n polynomial system P(z) with general coefficients cf(*), 
!     this subroutine will generate the FORTRAN subroutines of polynomial
!     evaluation and of Jacobian matrix evaluation for the system,
!     Their names are :
!     for the target system : zgetp(cf,z,p),  zgetdp(lda,cf,z,dp),
!     for the random system : zgetp1(cf,z,p),  zgetdp1(lda,cf,z,dp),
!     where
!           cf is the coefficients of p
!           z  is the varaible
!           p, dp are the polynomial value and Jaconbian matrix

!  .. Input and output

!     n     - The number of polynomials in the system
!
!     type  - type = 1 : nonlinear homotopy case. 
!                        To generate zgetp1.f90 and zgedp1.f90
!                  = 2 : linear homotopy case.
!                        To generate zgetp.f90 and zgetdp.f90

!     ia    - Each column of ia is a point in the support

!     ilow  - ilow(i) is the starting column number of the support
!             for i-th polynomial in ia

!     ihgh  - ihgh(i) is the ending column number of the support
!             for i-th polynomial in ia

!  .. Functions called ..

      INTEGER             len_trim
      CHARACTER*(len_i2c) i2c

!  .. Local scalar variables ..

      INTEGER              jcnt, i, i1, iz, j, nia
      CHARACTER*(len_name) ttl, filep, filedp, filename
      CHARACTER*(len_i2c)  c1, c2

!  .. Local scalar variables ..

c     INTEGER iidx(lda,0:mnop), iatmp(lda,mnop)
c     CHARACTER*(len_poly) a(lda,mnop) 
c     CHARACTER*(len_name) coef(mnop)

      INTEGER              iidx(lda,0:*), iatmp(lda,*)
      CHARACTER*(len_poly) a(lda,*) 
      CHARACTER*(len_name) coef(*)
      CHARACTER*200        ctmp

!     iidx  - Grouping information for Horner's method of polynomial evaluation

!     iatmp - Temporary storage for the columns of ia when generating
!             the Jacobian matrix.
!     coef  - Store the NAME of the coefficients.

!     a     - Work space for generating the FORTRAN subroutines

!  .. Executable Statements ..

      IF (type .EQ. 1) then
         filep  = 'zgetp1'
         filedp = 'zgetdp1'
      ELSE
         filep  = 'zgetp'
         filedp = 'zgetdp'
      ENDIF

!     to renew all the files and generate fixed coefficient
      OPEN(UNIT=1,FILE= filep(1:len_trim(filep))// '.f')
         WRITE(UNIT=1,FMT=100) filep(1:len_trim(filep))
 100     FORMAT (6x,'SUBROUTINE ', a6, '(cf,z,p)'
     &         / 6x,'IMPLICIT none'
     &         / 6x,'COMPLEX*16 cf(*), z(*), p(*)') 
      CLOSE(UNIT=1)

      OPEN(UNIT=1,file=filedp(1:len_trim(filedp))// '.f')
         WRITE(UNIT=1,FMT=200) filedp(1:len_trim(filedp))
 200     FORMAT (6x,'SUBROUTINE ', a7, '(lda,cf,z,dp)'
     &         / 6x,'IMPLICIT none' 
     &         / 6x,'INTEGER lda'
     &         / 6x,'COMPLEX*16 cf(*), z(*), dp(lda,*)') 
      CLOSE(UNIT=1)

      DO i = 1, n
!        For the i-th support
         nia = ihgh(i)-ilow(i)+1
         CALL group(lda,n,ihgh(i)-ilow(i)+1,ia(1,ilow(i)),iidx)
         DO i1 = 1, nia 
            c1 = i2c(i1+ilow(i)-1)
            coef(i1) = 'cf(' // c1(1:len_trim(c1)) // ')'
         ENDDO
         CALL gen1fun(lda,n,ia(1,ilow(i)),coef,iidx,a)
         c1 = i2c(i)
         ttl = 'p(' // c1(1:len_trim(c1)) // ')'
         filename=filep(1:len_trim(filep)) // '.f'
         CALL prnt(ttl,a(1,1),filename)

         DO iz = 1, n
!           To generate the derivative WRT z(iz) for each iz
            jcnt = 0
            DO j = ilow(i), ihgh(i)
               IF(ia(iz,j) .gt. 0) THEN
                  jcnt = jcnt + 1
                  DO i1 = 1, n
                     iatmp(i1,jcnt) = ia(i1,j)
                  ENDDO
                  iatmp(iz,jcnt) = ia(iz,j)-1
                  IF(ia(iz,j) .eq. 1) THEN
                     c1 = i2c(j)
                     coef(jcnt) = 'cf(' // c1(1:len_trim(c1)) // ')'
                  ELSE
                     c1 = i2c(j)
                     c2 = i2c(ia(iz,j))
                     coef(jcnt) = 'cf(' // c1(1:len_trim(c1)) // ')*'
     &                            // c2(1:len_trim(c2))
                  ENDIF
               ENDIF
            ENDDO
            IF (jcnt .eq. 0)  THEN
               a(1,1) = '0.0'
            ELSE
               nia = jcnt
               CALL group(lda,n,jcnt,iatmp(1,1),iidx)
               CALL gen1fun(lda,n,iatmp(1,1),coef,iidx,a)
            ENDIF
            c1 = i2c(i)
            c2 = i2c(iz)
            ttl = 'dp(' // c1(1:len_trim(c1)) // ',' 
     &            // c2(1:len_trim(c2)) // ')'
            filename = filedp(1:len_trim(filedp)) // '.f'
            CALL prnt(ttl,a(1,1),filename)
         ENDDO
      ENDDO

      filename = filep(1:len_trim(filep)) // '.f'
      OPEN(UNIT=1,FILE= filename , access='append')
         WRITE(UNIT=1,FMT=300)
 300     FORMAT (6X, 'RETURN' / 6X,'END' )
      CLOSE(UNIT=1)

      filename = filedp(1:len_trim(filedp)) // '.f'
      OPEN(UNIT=1,FILE= filename, access='append')
         WRITE(UNIT=1,FMT=400 )
 400     FORMAT (6X, 'RETURN' / 6X,'END' )
      CLOSE(1)

!     To generate zgetpr.f and zgetdpr.f

      OPEN(UNIT=1,FILE='zgetp.f')
      OPEN(UNIT=2,FILE='zgetpr.f')
         READ(1,FMT='(A)') ctmp
         WRITE(2,*) '      SUBROUTINE zgetpr(cf,z,p)'
         READ(1,FMT='(A)') ctmp
         WRITE(2,*) ctmp(1:len_trim(ctmp))
         READ(1,FMT='(A)') ctmp
         WRITE(2,*) '      DOUBLE PRECISION '//ctmp(17:len_trim(ctmp))
         i = 1
         DO WHILE(i .EQ. 1)
            READ(1,FMT='(A)') ctmp
            IF(ctmp(7:9) .EQ. 'END') i = 0
            WRITE(2,*) ctmp(2:len_trim(ctmp))
         ENDDO
      CLOSE(2)
      CLOSE(1)
      OPEN(UNIT=1,FILE='zgetdp.f')
      OPEN(UNIT=2,FILE='zgetdpr.f')
         READ(1,FMT='(A)') ctmp
         WRITE(2,*) '      SUBROUTINE zgetdpr(lda,cf,z,dp)'
         READ(1,FMT='(A)') ctmp
         WRITE(2,*) ctmp(1:len_trim(ctmp))
         READ(1,FMT='(A)') ctmp
         WRITE(2,*) ctmp(1:len_trim(ctmp))
         READ(1,FMT='(A)') ctmp
         WRITE(2,*) '      DOUBLE PRECISION '//ctmp(17:len_trim(ctmp))
         i = 1
         DO WHILE(i .EQ. 1)
            READ(1,FMT='(A)') ctmp
            IF(ctmp(7:9) .EQ. 'END') i = 0
            WRITE(2,*) ctmp(2:len_trim(ctmp))
         ENDDO
      CLOSE(2)
      CLOSE(1)

      RETURN
      END
