      subroutine dnonvtx(lda,n,ns,ia,jlow,jhgh,aa,jvtx,
     &                   b,xb,binv,sb,ib,ib0)
  
      implicit none 
 
*  .. Scalar Arguments ..
      integer          lda, n, ns
 
*  .. Array Arguments ..
      integer          ia(lda,*), jlow(*), jhgh(*), jvtx(*), 
     &                 ib(*), ib0(*)
      double precision aa(lda,*), binv(lda,*), b(*), xb(*), sb(*)

*  PURPOSE
*  =======
*     To locate all interior points in each support.
*
*  -- Writen on 6/15/96
*     Tangan Gao, Dept. of Math, Michigan State University
 
* lda    - (input) INTEGER
*          LEADING DIMENSION. It must be >= n+1
* n      - (input) INTEGER
*          NUMBER OF POLYNOMIALS IN THE SYSTEM
* ns     - (input) INTEGER
*          NUMBER of DIFFERENT SUPPORTS
* ia     - (input) INTEGER ARRAY OF DIMENSION lda by tnop
*          EACH COLUMN OF IA IS A POINT IN THE SUPPORTS
* jlow   - (input) INTEGER ARRAY OF DIMENSION lda
*          JLOW(I)-TH COLUMN OF IA IS THE FIRST POINT OF SUPPORT S(I)
* jhgh   - (input) INTEGER ARRAY OF DIMENSION lda
*          JHGH(I)-TH COLUMN OF IA IS THE LAST POINT OF SUPPORT S(I)
* jvtx   - (output) INTEGER ARRAY OF DIMENSION tnop
*          jvtx(i) = 1 if i-th column of ia is a vertex
*                    0 if i-th column of ia is a non-vertex
* ib     - (work)
* ib0    - (work)
* aa     - (work) DOUBLE PRECISION ARRAY OF DIMENSION lda by tnop+1
*          THE TABLE MATTRIX FOR SIMPLEX METHOD
* binv   - (work)
* b      - (work)
* sb     - (work)
* xb     - (work)
 
*  .. Local Variables ..
      integer           iseed, jstrt, jend, ptin, info, rnk, dim,
     &                  i, i1, j, itmp, k
      double precision  tmp
*  .. Function called ..
      double precision  dran2
 
*  .. Executable Code ..
 
*     initialize jvtx
      do i = jlow(1), jhgh(ns)
         jvtx(i) = 1
      enddo
 
*     to form the matrix for simplex method
 
*     choose a random point
      iseed = 0
      do i =1, n
          b(i) = dran2(iseed)
      enddo
 
*     lift the points
      do j = jlow(1), jhgh(ns)
         tmp=0.0d0
         do i1=1,n
            tmp=tmp+(ia(i1,j)-b(i1))*(ia(i1,j)-b(i1))
         enddo
         aa(1,j+1)=-sqrt(tmp)      ! j+1: shift a slot for the added 1st column
         do i1 = 1, n
            aa(i1+1,j+1) = ia(i1,j)
         enddo
         aa(n+2,j+1)=1.0d0
      enddo
 
      dim = n+2    
*     The row dimension of aa
 
      do i = 1, ns
*        for each support
 
         jstrt = jlow(i) 
*        the first column is the lifting direction
         jend = jhgh(i) + 1         ! +1: shift a slot for the added 1st column
 
         aa(1,jstrt) = 1.0d0
         do i1 = 2, n+2
            aa(i1,jstrt) = 0.0d0
         enddo
 
*        to uptrianglize the matrix aa
	 call duptri(lda,aa,jstrt,jend,dim,ib0,rnk)
 
         do j=1,rnk
            do i1=1,rnk
               binv(i1,j)=0.0d0
            enddo
            binv(j,j)=1.0d0
         enddo
 
         do j = jstrt+1, jend                                  ! for each point
 
            ptin=j
            do i1=1,rnk
               ib(i1)=0
            enddo
            do i1=1,rnk
               if(ib0(i1) .eq. j) then
                  ib(i1)=ptin
                  ptin=-ptin
                  goto 10
               endif
            enddo
 
 10         if(ptin .gt. 0) then
*              the j-th column will get into the basis.
*              to decide which column will get out of the basis
               do k=1, rnk
                  tmp=0.0d0
                  do i1=1,rnk
                     tmp=tmp+binv(k,i1)*aa(i1,j)
                  enddo
                  xb(k)=tmp                               !xb is work space now
               enddo
 
*              to find the biggest component from xb(1) to xb(rnk)
               tmp=1.0d-13
               itmp=0
               do k=2,rnk
                  if(abs(xb(k)) .gt. tmp .and. ib(k) .eq. 0) then
                     tmp=abs(xb(k))
                     itmp=k
                  endif
               enddo
               if(itmp .eq. 0) then
                  info=-1
                  stop   !!! it is impossible in this case
               endif

*              itmp-th column will get out of the basis

*              to find the new B^{-1} after itmp out and j in

               do k=1,itmp-1
                  xb(k)=xb(k)/xb(itmp)
               enddo
               do k=itmp+1,rnk
                  xb(k)=xb(k)/xb(itmp)
               enddo
               do i1=1,rnk
                  tmp=binv(itmp,i1)
                  do k=1,itmp-1
                     binv(k,i1)=binv(k,i1)-xb(k)*tmp
                  enddo
                  do k=itmp+1,rnk
                     binv(k,i1)=binv(k,i1)-xb(k)*tmp
                  enddo
                  binv(itmp,i1)=tmp/xb(itmp)
               enddo
               ib(itmp)=j
            endif
 
            do i1=1,rnk
               if(ib(i1) .ne. 0) then
                  xb(i1)=1.d0
               else
                  xb(i1)=0.0d0
                  ib(i1)=ib0(i1)
               endif
            enddo
 
            do i1=1,rnk
               b(i1)=aa(i1,j)
            enddo
 
            call drsmplx(lda,aa,b,xb,binv,sb,ib,
     &                   jstrt,jend,rnk,info)
            if(info .gt. 0) then
               if(abs(xb(info)) .gt. 1.0d-10) then
*                 It is a non-vertex
                  jvtx(j-1) = 0                ! -1: since the added 1st column
               endif
            endif
            do i1=1,rnk
               ib0(i1)=ib(i1)
            enddo
         enddo
      enddo

      return
      end
