      subroutine drsmplx(lda,a,b,xb,binv,sb,ib,
     &                   jstrt,jend,m,ieps)
*
      implicit none
*
***** scaler variables
*
      integer          lda, m, jstrt, jend, ieps
*
***** array variables 
*
      double precision a(lda,*), b(*), xb(*), binv(lda,*), sb(*)
      integer          ib(*)
*
***** purpose
*
*	this is the simplex for solving 
*               min -x(jstrt)
*               ax=b
*               x\ge 0
*
***** 
*       lda      (input) unchanged
*                leading dimension of a. must >2*n+1
*       n        the number of polynomials
*       jstrt    (input) unchanged
*                the starting column number
*       jend     (input) unchanged
*                the ending column number
*       m        (input) unchanged
*                the number of the constrant equations, =rnk
*	a(lda,*) (input) unchanged
*                the coefficient matrix. jstrt-column to jend-column
*                and 1-row to m-row are active in this program
*       b(m)     (input) unchanged
*                the constant term
*       xb(m)    (input/output)
*                the basic feasible solution
*       ib(m)    (input/output)
*                the index set for xb(m)
*       binv     (input/output)
*                the inverse matrix of the matrix corresponding to
*                the basic basis of xb(m)
*       sb       (workspace)
*                is binv*a(*,k), to decide which column will get out. 
*       ieps     (input/output)
*           =0 : the basis not contain jstrt-th column.
*           >0 : xb(ieps)=epsilon, the t-variable
*
***** workspace
      integer          i, j
      integer          ell, iter, k
      double precision eps, sigb, sum, tol, vkmin
*
      parameter(eps=1.0d-6,tol=1.0d-10)
*
**** executable code
*
      iter=0
      ieps=0
      do i=1,m
         if(ib(i) .eq. jstrt) then
            ieps=i
            goto 10
         endif
      enddo                !need to change
*
 10   continue
*        to compute u and find k
         if(ieps .eq. 0) then !!! $e_{n+1}$ is not in the basis
            k=jstrt
            vkmin=-1.0d0
         else !!! $e_{n+1}$ is in the basis
*           to find vkmin=v_k=min{c_i+a_i^Tu | i\notin I_b}
            k=0
            vkmin=1.0d30
            do i=jstrt,jend
*              to check if i is in ib
               do j=1,m
                  if(i .eq. ib(j)) goto 20
               enddo
*              if i is not in ib
               sum=0.0d0
               do j=1,m
                  sum=sum+a(j,i)*binv(ieps,j)
               enddo
               if(sum .lt. vkmin) then
                  vkmin=sum
                  k=i
               endif
 20         enddo
            if(vkmin .gt. -eps) then
c              print*, iter
               return  !!!have found the optimal solution
            endif
         endif
*
*        k-th column will get into the basis
*        to form sb
         do i=1,m
            sum=0.0d0
            do j=1,m
               sum=sum+binv(i,j)*a(j,k)
            enddo
            sb(i)=sum
         enddo
*
         ell=0
         sigb= 1.0d30
         do i=1,m
            if(sb(i) .gt. eps .and. (xb(i)/sb(i)) .lt. sigb) then
               sigb=(xb(i)/sb(i))
               ell=i
            endif
         enddo
*        ell-th column will get out of the basis and k-th column will
*        get into the basis.
*        to find the new $B^{-1}$
         sum=0.0d0
         do i=1,m
            sum=sum+binv(ell,i)*a(i,k)
         enddo
         if(abs(sum) .le. eps) then
            write(*,*) '$B^{-1}$ is singular'
            stop
         endif
*
         sum=1.0d0/sum
         do i=1,m
            binv(ell,i)=sum*binv(ell,i)
         enddo
         do j=1,m
            if(j .ne. ell) then
               sum=0.0d0
               do i=1,m
                  sum=sum+binv(j,i)*a(i,k)
               enddo
               do i=1,m
                  binv(j,i)=binv(j,i)-sum*binv(ell,i)
               enddo
            endif
         enddo
*
*        to form the new basic feasible solution
         do i=1,m
            sum=0.0d0
            do j=1,m
               sum=sum+binv(i,j)*b(j)
            enddo
            xb(i)=sum
         enddo
*
         if(ib(ell) .eq. jstrt) then
            ieps=0
*           the jstrt-th column is out of basis
         endif
         ib(ell)=k
         if(k .eq. jstrt) then
            ieps=ell
            if(xb(ieps) .gt. tol) then
c              print*, iter
               return
*              epsilon is already positive.
            endif
         endif
*
         iter=iter+1
*********************************
      goto 10
*
      end

