      subroutine duptri(lda,aa,jstrt,jend,dim,ib,rnk)
*
      implicit none
*
***** scalar variables
      integer          lda, jstrt, jend, dim, rnk
*
      integer          ib(*)
      double precision aa(lda,*)
*
***** purpose
*        to transform the matrix aa into upper triangular form
*        such that the first independent rnk columns of aa form an 
*        identity matrix and find its rank rnk.
*        ib(i)=k means that vector e_i is at k-th column
*
*       lda      (input) unchanged
*                the number of polynomials in the system
*       jstrt    (input) unchanged
*       jend     (input) unchanged
*                the columns from jstrt_th to jend-th in aa
*                are active in this subroutine
*       dim      (input) unchanged
*                the row dimension of the active part of aa
*       aa(2*n+1,*)  (input/output) changed
*                    the coefficient data for simplex method. 
*                    the jstrt-th column of aa is (1,0,...,0)^T
*       rnk      (output) changed
*                the row rank of the active part.
*
***** workspace
      integer          i, j, k, itmp
      double precision tmp
*
***** executable code
*
*       since the first column of aa is (1,0,...,0)^T
        ib(1)=jstrt
        do i=2,dim
           ib(i)=0
        enddo
        rnk=1
*       rnk will record the current size of the unit matrix
*
        k=jstrt+1
        do while(rnk .lt. dim .and. k .le. jend)
*          to find the biggest component from aa(rnk+1,k) to aa(dim,k)
           tmp=1.0d-15
           itmp=0
           do i=rnk+1,dim
              if(abs(aa(i,k)) .gt. tmp) then
                 tmp=abs(aa(i,k))
                 itmp=i
              endif
           enddo
*
           if(itmp .ne. 0) then
*             to change aa(*,k) into e_itmp
              do i=1,itmp-1
                 aa(i,k)=aa(i,k)/aa(itmp,k)
              enddo
              do i=itmp+1,dim
                 aa(i,k)=aa(i,k)/aa(itmp,k)
              enddo
              do j=k+1,jend
                 do i=1,itmp-1
                    aa(i,j)=aa(i,j)-aa(i,k)*aa(itmp,j)
                 enddo
                 do i=itmp+1,dim
                    aa(i,j)=aa(i,j)-aa(i,k)*aa(itmp,j)
                 enddo
                 aa(itmp,j)=aa(itmp,j)/aa(itmp,k)
              enddo
              if(itmp .ne. rnk+1) then
*                to change (rnk+1)-th row and itmp-th row of aa
                 do j=jstrt,jend
                    tmp=aa(rnk+1,j)
                    aa(rnk+1,j)=aa(itmp,j)
                    aa(itmp,j)=tmp
                 enddo
              endif
              do i=1,rnk
                 aa(i,k)=0.0d0
              enddo
              aa(rnk+1,k)=1.0d0
              do i=rnk+2,dim
                 aa(i,k)=0.0d0
              enddo
*
              rnk=rnk+1
              ib(rnk)=k
           endif
           k=k+1
        enddo
*
	return
	end
