      SUBROUTINE scaler(lda,n,ia,ap,ilow,ihgh,scl,
     &                  ib,a,binv,b,x,s,ap_)

      IMPLICIT NONE

!  .. Scalar variable ..

      INTEGER lda, n

!  .. Array variables ..

      INTEGER          ia(lda,*)
      INTEGER          ilow(*), ihgh(*)
      DOUBLE PRECISION scl(*)
      COMPLEX*16       ap(*)

!  .. Purpose ..

!     To scale both the coefficients and the variables of the system
!     to balance the coefficients.

!  .. Local scalar variables ..

      INTEGER          i, i1, j, ma, na, info
      DOUBLE PRECISION dtmp, dtmp1

!  .. Local array variables ..

      INTEGER          ib(*)
      DOUBLE PRECISION a(2*lda+1,*), binv(2*lda+1,*)
      DOUBLE PRECISION b(*), x(*), s(*)
      COMPLEX*16       ap_(*)

c     INTEGER       :: ib(2*n+1)
c     REAL(KIND=wp) :: a(2*n+1,2*SIZE(ia,2)), binv(2*n+1,2*n+1)
c     REAL(KIND=wp) :: b(2*SIZE(ia,2)), x(2*n+1), s(2*n+1)
c     COMPLEX(KIND=wp) :: ap_(SIZE(ia,2))

!     The method :
 
!     Let x=10^{\alpha} y. We want to solve
!            MIN M
!            1 <= 10^{\beta_i} |ap(j)| 10^{<\alpha,ia(1:n,j)>} <= M
!            for ilow(i)<=j<=ihgh(i), 1<=i<=n.
!     Where \beta, \alpha and M are variables. Taking log10 each side,
!     the question can be written as a linear programming problem:
!                MIN x(1) subject to  a x <= b.

!  .. Executable code ..

      DO i = 1, n
         dtmp = 1.0d50
         DO j = ilow(i), ihgh(i)
            dtmp = MIN(dtmp, ABS(ap(j)))
         ENDDO
         DO j = ilow(i), ihgh(i)
            ap_(j) = ap(j)/dtmp
         ENDDO
      ENDDO

!     To form the coefficent matrix a and constant terms b

      na = 2*n + 1
      ma = ihgh(n) -ilow(1) + 1
      DO i = 1, ma
         a(1,i) = -1.0d0
      ENDDO
      DO j = ilow(1), ihgh(n)
         DO i = 1, n
            a(i+1,j)   = ia(i,j)                                  ! ilow(1) = 1
         ENDDO
         DO i = n+2, 2*n+1
            a(i,j) = 0.0d0
         ENDDO
      ENDDO
      DO i = 1, n
         DO j = ilow(i), ihgh(i)
            a(n+1+i,j) = 1.0d0
         ENDDO
      ENDDO

      DO j = 1, ma
         b(j) = -LOG10(ABS(ap_(j)))
      ENDDO
      DO j = ma+1, 2*ma
         a(1, j) = 0.0d0
         DO i = 2, 2*n+1
            a(i,j) = -a(i,j-ma)
         ENDDO
         b(j) = -b(j-ma)
      ENDDO

      ma = 2*ma
      DO j = 1, na
         DO i = 1, na
            binv(i,j) = 0.0d0
         ENDDO
         binv(j,j) = 1.0d0
      ENDDO
      DO i = 1, na
         ib(i) = 0
      ENDDO
      x(1) = 0.0d0
      DO j = 1, ma
         x(1) = MAX(x(1),ABS(b(j)))
      ENDDO
      DO i = 2, na
         x(i) = 0.0d0
      ENDDO

      CALL alg2(a,b,x,binv,ib,na,ma,info,2*lda+1,s)      

      DO i = 1, n
         dtmp1 = 1.0d50
         DO j = ilow(i), ihgh(i)
            dtmp1 = MIN(dtmp1,ABS(ap(j)))
         ENDDO
         DO j = ilow(i), ihgh(i)
            dtmp = x(n+1+i)
            DO i1 = 1, n
               dtmp = dtmp + ia(i1,j)*x(i1+1)
            ENDDO
            ap(j) = (10.0d0**dtmp/dtmp1)*ap(j)
         ENDDO
         dtmp1 = 0.0d0
         DO j = ilow(i), ihgh(i)
            dtmp1 = MAX(dtmp1,ABS(ap(j)))
         ENDDO
         na   = LOG10(dtmp1)/2
         DO j = ilow(i), ihgh(i)
            ap(j) = ap(j)/10.0d0**na
         ENDDO
      ENDDO
      DO i = 1, n
         scl(i) = x(i+1)
      ENDDO

      RETURN
      END
