      PROGRAM main_prepro

      IMPLICIT NONE

      INCLUDE 'GENPDP/genpdp.h'

! genpdp.h defines the parameters len_i2c, len_poly, len_name which are
! the lengthes of some character variables used in the code genpdp.f.
! It is not necessary to change them since they are set large enough already.
! Detailed descriptions of the parameters are given in the file genpdp.h.

      INTEGER lda, mnop
      INCLUDE 'prepro.h'

! prepro.h was generated by suppcoef.m if you have executed suppcoef.m.
!          It defines the parameter lda and mnop.
! lda  must be greater than 2 + number of variables in the polynomial system
! mnop must be greater than lda + total number of points in the polynomial 
!      system since constant terms may be added

!  .. Purpose ..
 
! This is the preprocessor for main_solver.f90. It handles
! (1) Arrange the polynomial data in the form appropriate for main_solver.f90.
! (2) Generate the FORTRAN subroutines of polynomial and derivative evaluation.
! (3) Scale coefficients of polynomials if necessary.

!  .. Array variables ..

      INTEGER          n, ns
      INTEGER          ia(lda,mnop)
      INTEGER          ilow(lda), ihgh(lda)
      INTEGER          ia_vtx(lda,mnop)
      INTEGER          ilow_vtx(lda), ihgh_vtx(lda)
      INTEGER          irep(lda), iorder(mnop)
      DOUBLE PRECISION scl(lda)
      COMPLEX*16       ap(mnop), ap_scl(mnop)

!  .. For scaler.f

      INTEGER          ib2(2*lda+1)
      DOUBLE PRECISION a2(2*lda+1,2*mnop), binv2(2*lda+1,2*lda+1)
      DOUBLE PRECISION b2(2*mnop), x2(2*lda+1), s2(2*lda+1)
c     COMPLEX*16       ap_(mnop)  ! replaced by ap_new

!  .. For nonvtx.f

      INTEGER          ib(lda), ib0(lda)
      DOUBLE PRECISION aa(lda,mnop)
      DOUBLE PRECISION b(lda), xb(lda), s(lda)
      DOUBLE PRECISION binv(lda,lda)

!  .. For unmix_rrrn.f

      INTEGER    ia_new(lda,mnop)
      INTEGER    ilow_new(lda), ihgh_new(lda)
      INTEGER    iorder_new(mnop)
      COMPLEX*16 ap_new(mnop)
!     INTEGER    itmp(lda)  replaced by ib

!  .. For genpdp.f 

      INTEGER iidx(lda,0:mnop)
c     INTEGER iatmp(lda,mnop)  ! replaced by ia_new
      CHARACTER*(len_poly) ca(lda,mnop)
      CHARACTER*(len_name) ccoef(mnop)

!  .. Scalar variables ..

!      REAL              time(2)
      DOUBLE PRECISION  dtmp
      INTEGER           i, i1, j, j1
      INTEGER           coef_type
      CHARACTER*80      chtmp

!  .. Executable code  ..

!      CALL dtime(time)

      OPEN(10, FILE='trash.prepro')

      OPEN(2, FILE='data.supp')
         READ(2,*,END=3) n, ns

         READ(2,*,END=3) (irep(i),i=1,n)

         DO i = 1, n
            READ(2,*,END=3) ilow(i), ihgh(i)

            IF(ihgh(i)-ilow(i) .LT. 1) THEN 
               WRITE(10,*) i, '-th support has fewer than 2 points'
               STOP
            ENDIF
         ENDDO

         DO j = ilow(1), ihgh(n)
            READ(2,*,END=3) (ia(i,j),i = 1, n)
         ENDDO
      CLOSE(2)

      goto 4

 3    print*, 'The support data file is wrong'
      WRITE(10,*) 'The support data file is wrong'
      stop

!     To check whether there are points same in a support

 4    DO i = 1, n     
         DO j = ilow(i), ihgh(i)-1
            DO j1 = j+1, ihgh(i)
               DO i1 = 1, n
                  IF ( ia(i1,j1) .NE. ia(i1,j) ) GOTO 5
               ENDDO
               WRITE(10,*) j, ',', j1, '-th points are same'
               STOP
 5          ENDDO
         ENDDO
      ENDDO

!     To check whether it is a linear system

      DO i = ilow(1), ihgh(n)
         j = 0
         DO i1 = 1, n
            j = j + ia(i1,i)
         ENDDO
         IF (j .GT. 1) GOTO 10
      ENDDO

      WRITE(10,*) 'A linear system. Use a linear solver to solve it'
      STOP

 10   OPEN(UNIT=2,FILE='data.coef')
         READ(UNIT=2,FMT='(A)') chtmp
      CLOSE(UNIT=2)

      coef_type = 1 !real coefs.
      IF (index(chtmp,',') .GT. 0) coef_type = 2 !complex coefs.

      OPEN(2, FILE='data.coef')
         IF(coef_type .eq. 1) THEN
            DO j = ilow(1), ihgh(n)
               read(2,*) dtmp
               ap(j) = cmplx(dtmp,0.0d0)
            ENDDO
         ELSE
            DO j = ilow(1), ihgh(n)
               read(2,*) ap(j)
            ENDDO
         ENDIF
      CLOSE(2)

      CALL prepro(lda,n,ns,ia,ilow,ihgh,ia_vtx,ilow_vtx,ihgh_vtx,
     &            irep,iorder,scl,ap,ap_scl,
     &            ia_new,ilow_new,ihgh_new,iorder_new,ap_new,  !workspace
     &            ib,ib0,aa,b,xb,s,binv,
     &            ib2,a2,binv2,b2,x2,s2,
     &            iidx,ca,ccoef)

      write(10,*) '# of polynomials: ', n
      write(10,*) '# of different supports:', ns
      write(10,*) '# of times supports repeat:', (irep(i),i=1,ns)

      OPEN(2, FILE='SRC/SOLVER/dimension.h')
         WRITE(2,*) '      PARAMETER(lda =',n+2,', tnop =', 
     &               ihgh(n)+2, ')'
         WRITE(2,20) '! This file is generated by PREPRO/main_prepro.f'
      CLOSE(2)

 20   FORMAT(80a)

      OPEN(2, FILE='SRC/SOLVER/mixvol.h')

!        use the total degree of the system as the upper bound for 
!        its mixed volume. The file mixvol.h is used for SOLVER/main_mixvol.f

         coef_type = 1   ! use to record the total degree
         DO i = 1, n
            j1 = 0
            DO j = ilow(i), ihgh(i)
               dtmp = 0
               DO i1 = 1, n
                  dtmp = dtmp + ia(i1,j)
               ENDDO
               i1 = dtmp + 0.01
               j1 = MAX(j1,i1)
            ENDDO
            coef_type = coef_type*j1
         ENDDO

! To estimate the upper bound for the possible edges in each support

         do i = 1, ns
            dtmp = 1
            do i1 = ihgh_vtx(i)-ilow_vtx(i)+1-irep(i),
     &                                ihgh_vtx(i)-ilow_vtx(i)+1
               dtmp = dtmp*i1
            enddo
            do i1 = 2, irep(i)+1
               dtmp = dtmp/i1
            enddo
            j = j + dtmp
         enddo
         j = j + 5

         coef_type = max(coef_type+2,400)
         coef_type = min(coef_type,1000000)

         WRITE(2,*) '      PARAMETER(maxmvl =', coef_type,
     &              ', maxedges =', j, ')'
         WRITE(2,20) '! This file is generated by PREPRO/main_prepro.f'
         WRITE(2,20) '! maxmvl = 2 + the total degree of the system'
      CLOSE(2)

      OPEN(2, FILE='for.mixvol')
         write(2,*) n, ns
         write(2,*) (irep(i),i=1,ns)
         do i = 1, n
            write(2,*) ilow_vtx(i), ihgh_vtx(i)
         enddo
         do j = ilow_vtx(1), ihgh_vtx(n)
            write(2,*) (ia_vtx(i,j),i=1,n)
         enddo
         do i = ilow_vtx(1), ihgh_vtx(n)
            write(2,*) iorder(i)
         enddo
         do i = 1, n
            write(2,*) ilow(i), ihgh(i)
         enddo
         do j = ilow(1), ihgh(n)
            write(2,*) (ia(i,j),i=1,n)
         enddo
         do i = ilow(1), ihgh(n)
            write(2,*) ap_scl(i)
         enddo
         do i = 1, n
            write(2,*) scl(i)
         enddo
         do i = ilow(1), ihgh(n)
            write(2,*) ap(i)
         enddo
      CLOSE(2)

!      CALL dtime(time)
!      write(10,*) 'CPU time for this run is ', time(1)
      CLOSE(10)

      STOP
      END
