      SUBROUTINE prepro(lda,n,ns,ia,ilow,ihgh,ia_vtx,ilow_vtx,ihgh_vtx,
     &                  irep,iorder,scl,ap,ap_scl,
     &                  ia_new,ilow_new,ihgh_new,iorder_new,ap_new,
     &                  ib,ib0,aa,b,xb,s,binv,
     &                  ib2,a2,binv2,b2,x2,s2,
     &                  iidx,ca,ccoef)

      IMPLICIT NONE

      INCLUDE 'GENPDP/genpdp.h'

!  .. Purpose ..
 
! This is the preprocessor for main_solver.f90. It handles
! (1) Arrange the polynomial data in the form appropriate for main_solver.f90.
! (2) Generate the FORTRAN subroutines of polynomial and derivative evaluation.
! (3) Scale coefficients of polynomials if necessary.

!  .. Array variables ..

      INTEGER          lda, n, ns
      INTEGER          ia(lda,*)
      INTEGER          ilow(*), ihgh(*)
      INTEGER          ia_vtx(lda,*)
      INTEGER          ilow_vtx(*), ihgh_vtx(*)
      INTEGER          irep(*), iorder(*)
      DOUBLE PRECISION scl(*)
      COMPLEX*16       ap(*), ap_scl(*)

!  .. For scaler.f

      INTEGER          ib2(*)
      DOUBLE PRECISION a2(2*lda+1,*), binv2(2*lda+1,*)
      DOUBLE PRECISION b2(*), x2(*), s2(*)

!  .. For nonvtx.f

      INTEGER          ib(*), ib0(*)
      DOUBLE PRECISION aa(lda,*)
      DOUBLE PRECISION b(*), xb(*), s(*)
      DOUBLE PRECISION binv(lda,*)

!  .. For unmix_rrrn.f

      INTEGER    ia_new(lda,*)
      INTEGER    ilow_new(*), ihgh_new(*)
      INTEGER    iorder_new(*)
      COMPLEX*16 ap_new(*)
!     INTEGER    itmp(lda)  replaced by ib

!  .. For genpdp.f 

      INTEGER iidx(lda,0:*)
      CHARACTER*(len_poly) ca(lda,*)
      CHARACTER*(len_name) ccoef(*)

!  .. Scalar variables ..

      INTEGER           i, i1, icnt, j, j1, flag

!  .. Executable code  ..

!  .. To sort the points of each support in lexicographicaly decending order
!     and rearrange the coefficient in the same order

      CALL sort_supp(lda,n,ia,ap,ilow,ihgh)

!     To scale the coefficients

      DO j = ilow(1), ihgh(n)
         ap_scl(j) = ap(j)
      ENDDO

      CALL scaler(lda,n,ia,ap_scl,ilow,ihgh,scl,
     &            ib2,a2,binv2,b2,x2,s2,ap_new)

!     To add the constant terms if necessary

      DO i = 1, n
         ib(i) = 0
      ENDDO
      icnt = 0
      DO i = 1, n
         DO i1 = 1, n
            IF (ia(i1,ihgh(i)) .NE. 0) THEN
               ib(i) = 1
               icnt = icnt + 1
               GOTO 17
            ENDIF
         ENDDO
 17   ENDDO

c     icnt = 0
      IF ( icnt .EQ. 0 ) GOTO 20

      WRITE(10,*) '# of constant terms added : ', icnt
      DO i = n, 2, -1
         IF (ib(i) .EQ. 1) THEN

            DO i1 = 1, n                           ! The term added
               ia(i1,ihgh(i)+icnt) = 0
            ENDDO
            ap(ihgh(i)+icnt) = 0.0d0
            ap_scl(ihgh(i)+icnt) = 0.0d0
            DO j1 = ihgh(i), ilow(i), -1               ! shift the points
               DO i1 = 1, n
                  ia(i1,j1+icnt-1) = ia(i1,j1)
               ENDDO
               ap(j1+icnt-1) = ap(j1)
               ap_scl(j1+icnt-1) = ap_scl(j1)
            ENDDO

            ilow(i) = ilow(i) + icnt - 1
            ihgh(i) = ihgh(i) + icnt
            icnt = icnt - 1
         ELSE
            DO j1 = ihgh(i), ilow(i), -1
               DO i1 = 1, n
                  ia(i1,j1+icnt) = ia(i1,j1)
               ENDDO
               ap(j1+icnt) = ap(j1)
               ap_scl(j1+icnt) = ap_scl(j1)
            ENDDO
            ilow(i) = ilow(i) + icnt 
            ihgh(i) = ihgh(i) + icnt
         ENDIF
      ENDDO
      IF (ib(1) .EQ. 1) THEN
         DO i = 1, n
            ia(i,ihgh(1)+icnt) = 0
         ENDDO
         ap(ihgh(1)+icnt) = 0.0d0
         ap_scl(ihgh(1)+icnt) = 0.0d0
         ihgh(1) = ihgh(1) + icnt
      ENDIF

!  .. To locate all non-vertex points in each support ..

 20   CALL dnonvtx(lda,n,n,ia,ilow,ihgh,aa,iorder,
     &             b,xb,binv,s,ib,ib0)

!     To reorder the supports if necessary

      do i = 1, n
         ilow_vtx(i) = 0       ! temp variable, # of vertex pts in each support
         do j = ilow(i), ihgh(i)
            if (iorder(j) .eq. 1) ilow_vtx(i) = ilow_vtx(i) + 1
         enddo
      enddo

c     print*, (ilow_vtx(i),i=1,n)

      flag = 0
      do i = 1, n
         ib(i) = 0
      enddo
      do i = 1, n
         j1 = 1000000
         do j = 1, n
            if (ib(j) .eq. 0 .AND. ilow_vtx(j) .lt. j1) then
               j1 = ilow_vtx(j)
               ib0(i) = j
            endif
         enddo
         if (ib0(i) .ne. i) flag = 1
         ib(ib0(i)) = 1
      enddo

      write(10,*) 'New order of the polynomials :', (ib0(i),i=1,n)

      if (flag .eq. 1) then

!        To switch ia, ilow, ihgh, ap, ap_scl, iorder

         icnt = 0
         do i = 1, n
            do j = ilow(ib0(i)), ihgh(ib0(i))
               icnt = icnt + 1
               ap_new(icnt) = ap_scl(j)
            enddo
         enddo
         do j = 1, icnt
            ap_scl(j) = ap_new(j)
         enddo

         icnt = 0
         do i = 1, n
            do j = ilow(ib0(i)), ihgh(ib0(i))
               icnt = icnt + 1
               do i1 = 1, n
                  ia_new(i1,icnt) = ia(i1,j)
               enddo
               ap_new(icnt)       = ap(j)
               iorder_new(icnt)   = iorder(j)
            enddo
            ib(i) = icnt
         enddo
         do j = 1, icnt
            do i = 1, n
               ia(i,j) = ia_new(i,j)
            enddo
            ap(j)     = ap_new(j)
            iorder(j) = iorder_new(j)
         enddo

         ilow(1) = 1
         ihgh(1) = ib(1)
         do i = 2, n
            ilow(i) = ihgh(i-1) + 1
            ihgh(i) = ib(i)
         enddo
      endif

      icnt = 0
      DO i = 1, n
         DO j = ilow(i), ihgh(i)
            IF (iorder(j) .eq. 1) THEN
               icnt = icnt + 1
               DO i1 = 1, n
                  ia_vtx(i1,icnt) = ia(i1,j)
               ENDDO
               iorder(icnt) = j       

!              For the vertex system, iorder(i) = j means that 
!              the current i-th point is the j-th point in the original system

            ENDIF
         ENDDO
         ihgh_vtx(i) = icnt
      ENDDO
      ilow_vtx(1)   = 1
      DO i = 2, n
         ilow_vtx(i) = ihgh_vtx(i-1) + 1
      ENDDO

      write(10,*) 'Total # of points     : ',ihgh(n)
      write(10,*) '# of nonvertex points : ',ihgh(n)-ihgh_vtx(n)
      write(10,*) '# of vertex points    : ',ihgh_vtx(n)-ilow_vtx(1)+1

!     To find whether there are some supports same in the vertex system

      DO i = 1, n
         irep(i) = -1
      ENDDO
      ns   = 0
      DO i = 1, n-1
        IF(irep(i) .LT. 0) THEN
          ns = ns + 1
          DO j = i+1, n
            IF(irep(j) .LT. 0) THEN 
              IF (ihgh_vtx(j)-ilow_vtx(j) .NE. 
     &            ihgh_vtx(i)-ilow_vtx(i) ) GOTO 25
              DO j1 = ilow_vtx(i), ihgh_vtx(i)
                DO i1 = 1, n
                  IF (ia_vtx(i1,j1) .NE. 
     &              ia_vtx(i1,j1+(ilow_vtx(j)-ilow_vtx(i)))) GOTO 25 
                ENDDO
              ENDDO
              irep(j) = i
            ENDIF
 25       ENDDO
        ENDIF
      ENDDO

      IF(irep(n) .LT. 0) THEN
         ns = ns + 1
      ENDIF

!     irep(i) = -1 : the i-th support is the base support
!             = j  : the i-th support is the same as the j-th base support

!     For the unmixed system, rearrange the supports in appropriate order

      IF(ns .LT. n) THEN
         CALL unmix_rrrng(lda,n,ia,ilow,ihgh,ia_vtx,ilow_vtx,
     &                    ihgh_vtx,ap,ap_scl,irep,iorder,
     &                    ia_new,ilow_new,ihgh_new,iorder_new,   ! workspace
     &                    ap_new,ib)
      ELSE
         ns = n
         DO i = 1, n
            irep(i) = 1 
         ENDDO
      ENDIF

!     To write polynomial information to data file for main_poly.f

      CALL genpdp(lda,n,ia,ilow,ihgh,2,
     &            iidx,ia_new,ca,ccoef)
      CALL genpdp(lda,n,ia_vtx,ilow_vtx,ihgh_vtx,1,
     &            iidx,ia_new,ca,ccoef)

      RETURN
      END
