      SUBROUTINE dhshld_slv(lda,n,a,b,info)

      IMPLICIT none

!  .. Scalar Variables ..

      INTEGER lda, n,info

!  .. Array Variables ..

      DOUBLE PRECISION a(lda,*), b(*)

!  .. Local array avriables ..

      DOUBLE PRECISION v(200)

!  .. PURPOSE ..

!   (1) Use Householder transformation to change A into QR and
!       b into Qb. Q is not saved. R repalces A and Qb replace b
!       The household matrix is chosen according to the way suggested
!       in "Matrix Computations" by G.H. Golub & C.F. Van Loan
!   (2) Solve A x = b if A is nonsingular.

!  .. Input/output ..

! a(n,n) On entry, a is the coefficietn matrix;
!        on exit, a is R. 
! b(n)   On entry, b is the right side of a*x=b; 
!        on exit, b is the solution if info = 0.
! info = 0 : everything is OK
!      = 1 : a is not of full rank

!  .. Local Variables ..

      INTEGER          i, i1, j
      DOUBLE PRECISION eps, sigma, nu, beta, dtmp

!  .. Executable Code ..

      info = 1
      eps  = 1.0d-7

      DO i = 1, n-1

!        For the vector a(i:n,i), compute v, beta

         dtmp = a(i,i)
         v(i) = 1.0d0
         DO i1 = i+1, n
            v(i1) = a(i1,i)   !! scaling of v <- v/||v|| is not applied here
         ENDDO
         sigma = 0.0d0
         DO i1 = i+1, n
            sigma = sigma + v(i1)*v(i1)
         ENDDO
         nu = SQRT(dtmp**2 + sigma)
         IF(nu .LT. eps) RETURN
         IF(sigma .EQ. 0.0d0) THEN
            GOTO 10
         ELSE
            IF(dtmp .LE. 0.0d0) THEN
               dtmp = dtmp - nu
            ELSE
               dtmp = -sigma/(dtmp + nu)
            ENDIF
            beta  = 2.0d0*dtmp**2/(sigma + dtmp**2)
            DO i1 = i+1, n
               v(i1) = v(i1)/dtmp
            ENDDO
         ENDIF

!        To apply the transformation to a and b

         a(i,i) = nu
         DO j = i+1, n
            dtmp = 0.0d0
            DO i1 = i,n
               dtmp = dtmp + v(i1)*a(i1,j)
            ENDDO
            dtmp = beta*dtmp
            DO i1 = i, n
               a(i1,j) = a(i1,j) - dtmp*v(i1)
            ENDDO
         ENDDO

         dtmp = 0.0d0
         DO i1 = i,n
            dtmp = dtmp + v(i1)*b(i1)
         ENDDO
         dtmp = beta*dtmp
         DO i1 = i, n
            b(i1) = b(i1) - dtmp*v(i1)
         ENDDO
 10   ENDDO
      IF(ABS(a(n,n)) .LT. eps) RETURN

      b(n) = b(n)/a(n,n)
      DO j = n-1, 1, -1
         DO i1 = 1, j
            b(i1) = b(i1) - a(i1,j+1)*b(j+1)
         ENDDO
         b(j)   = b(j)/a(j,j)
      ENDDO
      info = 0

      RETURN
      END
