      SUBROUTINE fnl_iter(lda,n,ap,x,x0,info,dhz,h,hnrm)

      IMPLICIT none

      include '../global.h'

!  .. Scalar Arguments ..

      INTEGER lda, info, n
      DOUBLE PRECISION hnrm

!  .. Array Arguments ..

      COMPLEX*16 ap(*), x(*), x0(*), h(*), dhz(lda,*)

!  .. PURPOSE ..

!	Final Newton's iterations at t=1 to refine the roots.

!  .. Input and output ..

! ap     The coefficients of the system.
! x      The initial point for Newton's iteration on input.
!        The last point from the Newton's iteration on exit.
! hnrm   1-norm of the last function value.
! info=0 Newton's iteration converges.
!     =1 The Jaconbian matrix is singular.
!     =2 Newton's iteration diverges.

! dhz    Jacobian matrix for Newton's method.
! h      function values for Newton's method.

!  .. Local Variables ..

      INTEGER          i, iter
      DOUBLE PRECISION min_nrmh, diffx

!  .. Executable Code ..

      info = 0
      iter = 0
 
      CALL zgetp(ap,x,h)
 
      hnrm = 0.0d0
      DO i = 1, n
         hnrm = MAX(hnrm,ABS(DBLE(h(i)))+ABS(DIMAG(h(i))))
      ENDDO

      IF (hnrm .LT. 1.0d-14) GOTO 30

      min_nrmh = hnrm
      DO i = 1, n
         x0(i) = x(i)
      ENDDO

      DO WHILE (iter .LT. gmax_nwtn) 
 
         CALL zgetdp(lda,ap,x,dhz)
         CALL cqrdc_slv(lda,n,dhz,h,info)

         diffx = 0.0d0
         DO i = 1, n
            diffx = MAX(diffx, ABS(DBLE(h(i)))+ABS(DIMAG(h(i))))
         ENDDO

         IF(info .EQ. 1 .OR. diffx .gt. 1.0d3) THEN
            hnrm = min_nrmh
            DO i = 1, n
               x(i) = x0(i)
            ENDDO
            GOTO 30
         ENDIF

         DO i = 1, n
            x(i) = x(i) - h(i)
         ENDDO
 
         CALL zgetp(ap,x,h)

         hnrm = 0.0d0
         DO i = 1, n
            hnrm = MAX(hnrm,ABS(DBLE(h(i)))+ABS(DIMAG(h(i))))
         ENDDO

         IF (hnrm .lt. min_nrmh) THEN
            min_nrmh = hnrm
            DO i = 1, n
               x0(i) = x(i)
            ENDDO
         ENDIF

         IF (hnrm .LT. 1.0d-14 ) THEN
            GOTO 20
         ENDIF

         iter = iter + 1

      ENDDO

 20   IF (min_nrmh .lt. hnrm) THEN
         hnrm = min_nrmh
         DO i = 1, n
            x(i) = x0(i)
         ENDDO
      ENDIF
 30   IF(debug) THEN
         WRITE(10,*) '  The max function value : ', hnrm
      ENDIF

      RETURN
      END                
