      SUBROUTINE z1step(lda,n,ilow,ihgh,aq,ap,cft,p_old,v_old,
     &                  p_new,v_new,t_new,stp_new,stp_c,ntimes,
     &                  info,alpha1,yn_alpha,yn_infnty,
     &                  dhz,h,p0,cf,alpha2)

      IMPLICIT none

      include '../global.h'

!  .. Scalar Arguments

      INTEGER          lda, n, info, ntimes
      DOUBLE PRECISION t_new, stp_new, stp_c
      LOGICAL          yn_alpha, yn_infnty

!  .. Array Arguments

      INTEGER          ilow(*), ihgh(*)
      COMPLEX*16       ap(*), aq(*), cft(*),
     &                 p_old(*), p_new(*), v_old(*), v_new(*)
      DOUBLE PRECISION alpha1(*)

!  .. Local array variables ..

      COMPLEX*16       dhz(lda,*), h(*), p0(*), cf(*)
      DOUBLE PRECISION alpha2(*)

!  .. PURPOSE ..

!       this is one step of following a curve from two previous
!       points known until find next point on the curve.
!       after find the point, it will renew p_*,v_*,stp_*.

!  .. Input and output ..

! p_old, p_new  The last two consecutive  points known on the curve. 
! v_old, v_new  The tangent vectors at the point p_old, p_new.
! stp_new    The step size to get p_new from p_old.
! stp_c      The current step size to predict the next point.
! info= 2 Newton's iteration divergens.
!     = 1 Coefficient matrix dhz is singular.
!     = 0 Newton's iteration converges.
!     = -1 Got a root at t=1
!
! dhz     Jacobian matrix for Newton's method
! h       Function values for Newton's method
! p0      The points generated by Newton's method

!  .. Local Variables ..

      INTEGER          i, iter, icnt
      DOUBLE PRECISION nrm_p0, nrm_p1, nrm_dif, nrm_pnew
      DOUBLE PRECISION dznrm2
      LOGICAL          yn_ngtv

!  .. Executable code ..

      icnt = 0

      IF(t_new+stp_c .ge. 1.0d0-gstp_min) THEN       ! iteration on level t=1
         stp_c = 1.0d0 - t_new
         CALL zhermite3(n,p_old,v_old,stp_new,p_new,v_new,stp_c,p0)
         CALL znwtnat1(lda,n,ap,p0,info,dhz,h)
         IF(info .eq. 0) THEN                         ! p_new is the solution
            DO i = 1, n
               p_new(i) = p0(i)
            ENDDO
            info = -1
            RETURN
         ELSE
            stp_c = stp_c*0.68d0
         ENDIF
      ENDIF

!     To predict next point p0 near the curve

 10   CALL zhermite3(n,p_old,v_old,stp_new,p_new,v_new,stp_c,p0)

      DO i = ilow(1), ihgh(n)
         cf(i) = (1-(t_new+stp_c))*aq(i) + (t_new+stp_c)*ap(i)
      ENDDO
      CALL zgetp(cf,p0,h)
      nrm_p0 = dznrm2(n,h)

      iter = 0
      DO WHILE (iter .le. gmax_nwtn) 

!        One step of Newton's iteration

         CALL zgetdp(lda,cf,p0,dhz)
         CALL cqrdc_slv(lda,n,dhz,h,info)
         IF(info .ne. 0) RETURN                         !! dhz is singular
c        IF(info .eq. 1) RETURN                         !! dhz is singular
         DO i = 1, n
            p0(i) = p0(i) - h(i)
         ENDDO
         nrm_dif = dznrm2(n,h)

         CALL zgetp(cf,p0,h)
         nrm_p1 = dznrm2(n,h)

         nrm_pnew = 1.0d0
         DO i = 1, n
            nrm_pnew = max(nrm_pnew,ABS(DBLE(p0(i)))+ABS(DIMAG(p0(i))))
         ENDDO
         nrm_pnew = MIN(nrm_pnew,1.0d5)   !! To keep dtmp not too big
         IF(nrm_p1/nrm_pnew .lt. geps_trace .OR. 
     &      nrm_dif .lt. geps_trace*100 ) THEN

!           To store the information for next step

            DO i = 1, n
               p_old(i) = p_new(i)
            ENDDO

            DO i = 1, n
               p_new(i) = p0(i)
            ENDDO
            t_new = t_new + stp_c

!           To test whether the curves tends to be divergent

            IF(((.NOT. yn_infnty) .AND. stp_c .LT. 0.00001            
     &               .AND. t_new .GT. 0.9999)              .OR.  
     &         ((.NOT. yn_infnty) .AND. stp_c .LT. 0.00001          
     &               .AND. t_new .GT. 0.99                          
     &               .AND. nrm_pnew .GT. 1.0e2) ) THEN
               yn_ngtv = .FALSE.
               DO i = 1, n
                  alpha2(i) =LOG(ABS(p_new(i)/p_old(i)))     
     &                       /LOG((1-t_new)/(1-t_new+stp_c))
                  IF(alpha2(i) .LT. -8.) yn_ngtv = .TRUE.
c                 IF(alpha2(i) .LT. -0.1) yn_ngtv = .TRUE.    ! old 522
c                 how negative to be cosidered divergent curve?
c                 if no divergent curve, pick a more negative number;
c                 if there are divergent curves, set it at -0.1
c                 (too close to 0 makes a lot trouble!)
               ENDDO
               IF(yn_ngtv) THEN
                  IF(yn_alpha) THEN
                     DO i = 1, n
                        IF(abs(alpha1(i)-alpha2(i)) .GT. 0.01) THEN
                           yn_alpha = .FALSE.
                           GOTO 33
                        ENDIF
                     ENDDO
 33                  IF(yn_alpha) THEN
                        yn_infnty = .TRUE.

                        IF(debug) THEN
                          WRITE(10,*) '  Divergent direction : '
                          DO i = 1, n
                             WRITE(10,*) alpha2(i)
                          ENDDO
                        ENDIF
!!!!!!!!!!!
                        info = 3
                        return
!!!!!!!!!!!
                     ENDIF
                  ENDIF
                  DO i = 1, n
                     alpha1(i) = alpha2(i)
                  ENDDO
                  yn_alpha = .TRUE.
               ENDIF 
            ENDIF

            DO i = 1, n
               v_old(i) = v_new(i)
            ENDDO

!           To get the new tangent vector at p_new

            CALL zgetdp(lda,cf,p_new,dhz)
            CALL zgetp(cft,p_new,v_new)
            CALL cqrdc_slv(lda,n,dhz,v_new,info)
            IF(info .ne. 0) RETURN                         !! dhz is singular
c           IF(info .eq. 1) RETURN                         !! dhz is singular
            info = 0
            RETURN
         ENDIF

         IF((iter .eq. 0 .OR. iter .eq. 1) .AND. 
     &       nrm_p1/nrm_p0 .ge. grsdl_fctr) GOTO 20

         nrm_p0  = nrm_p1
         iter    = iter + 1
      ENDDO

 20   stp_c = stp_c*0.5d0
      IF(stp_c .LT. gstp_min) THEN
         info = 2
         RETURN
      ENDIF

      GOTO 10

      END
