      SUBROUTINE z1ststp(lda,n,ilow,ihgh,aq,ap,cft,p_old,v_old,
     &                   stp_c,p_new,v_new,info,
     &                   dhz,h,cf)

      IMPLICIT none

      include '../global.h'

!  .. Scalar Arguments

      INTEGER          lda, n, info
      DOUBLE PRECISION stp_c
  
!  .. Array Arguments

      INTEGER          ilow(*), ihgh(*)
      COMPLEX*16       ap(*), aq(*), cft(*),
     &                 p_old(*), p_new(*), v_old(*), v_new(*)

!  .. Local array variables ..

      COMPLEX*16  dhz(lda,*), h(*), cf(*)

!  .. PURPOSE ..

!       this is first step of following a curve from initial
!       point known until to find next point on the curve.
!       after find the point, the two points known woll be stored
!       in  p_old and p_new. the corresponding tangent vector will
!       be in v_old and v_new.

!  .. Input and output ..

! p_old, p_new   The last two points known on the curve. 
! v_old, v_new   The tangent vector of the curve at the point p_old, p_new.
! stp_c   The current step size to predict the next point
! info= 2 Newton's iteration divergens.
!     = 1 Coefficient matrix dhz is singular.
!     = 0 Newton's iteration converges.
!
! dhz     Jacobian matrix at p_new. 
! h       Function values at p_new.

!  .. Local Variables

      INTEGER          i, iter
      DOUBLE PRECISION nrm_p0, nrm_p1, nrm_dif, dtmp
      DOUBLE PRECISION dznrm2

!  .. Executable Code ..

!     To predict next point near the curve(linear prediction)

 10   DO i = 1, n
         p_new(i) = p_old(i) + stp_c*v_old(i)
      ENDDO

      DO i = ilow(1), ihgh(n)
         cf(i) = (1-stp_c)*aq(i) + stp_c*ap(i)
      ENDDO
 
      CALL zgetp(cf,p_new,h)
      nrm_p0 = dznrm2(n,h)

      iter = 0
      DO WHILE(iter .le. gmax_nwtn) 

!        One step of Newton's iteration

         CALL zgetdp(lda,cf,p_new,dhz)
         CALL cqrdc_slv(lda,n,dhz,h,info)
         IF(info .eq. 1) RETURN                            !! dhz is singular
         DO i = 1, n
            p_new(i) = p_new(i) - h(i)
         ENDDO
         nrm_dif = dznrm2(n,h)

         CALL zgetp(cf,p_new,h)
         nrm_p1 = dznrm2(n,h)

         dtmp = 1.0d0
         DO i = 1, n
            dtmp = MAX(dtmp,ABS(DBLE(p_new(i)))+ABS(DIMAG(p_new(i))))
         ENDDO
         IF(nrm_p1/dtmp .lt. geps_trace .OR. 
     &      nrm_dif .lt. geps_at1*100 ) THEN

!           Accept p_new and to get the new tangent vector at p_new

            CALL zgetdp(lda,cf,p_new,dhz)
            CALL zgetp(cft,p_new,v_new)
            CALL cqrdc_slv(lda,n,dhz,v_new,info)
            IF(info .eq. 1) RETURN                         !! dhz is singular

            info = 0
            RETURN
         ENDIF

         IF(nrm_p1/nrm_p0 .ge. grsdl_fctr) GOTO 20  ! residual ratio is too big

         nrm_p0 = nrm_p1
         iter   = iter + 1
      ENDDO

  20  stp_c = stp_c*0.5d0
      IF(stp_c .lt. gstp_min) THEN
         info = 2
         RETURN
      ELSE
         GOTO 10
      ENDIF

      END
