      SUBROUTINE znpath(lda,n,ilow,ihgh,aq,ap_scl,scl,ap,
     &                  roots0,ntimes,nrts,roots1,
     &                  ap_real,cft,p0,p1,
     &                  v_old,v_new,dhz,alpha,h,cf,alpha2,
     &                  p2,rdhz,hnrm,dhegn)

      IMPLICIT none

      include '../global.h'

!  .. Scalar Arguments ..

      INTEGER lda, n, nrts

!  .. Arrany Arguments ..

      INTEGER ilow(*), ihgh(*), ntimes(*)
      COMPLEX*16 ap_scl(*), aq(*), ap(*), 
     &           roots0(lda,*), roots1(lda,*)
      DOUBLE PRECISION scl(*), hnrm(*), dhegn(*)

!  .. Local array variables ..

      DOUBLE PRECISION ap_real(*)
      COMPLEX*16       cft(*), p0(*), p1(*)

      COMPLEX*16       v_old(*), v_new(*), dhz(lda,*),
     &                 h(*), cf(*), p2(*)
      DOUBLE PRECISION alpha(*), alpha2(*)

      DOUBLE PRECISION rdhz(lda,*)

!  .. PURPOSE ..

!  	znpath is to follow all homotopy curves for a linear homotopy

!  .. Input and output ..

! aq      The coefficients of the random system.
! ap_scl  The coefficients of the target system.
! roots0  The initial roots for the homotopy at t=0.
! ntimes  ntimes(j) = 1 : the j-th initial root at t=0 is available.
!                   = 0 : the j-th initial root at t=0 is NOT available.
! roots1  The roots of the target system(at t=1)
! nrts    On entry, nrts is the mixed volume;
!         on exit, nrts is the # of roots found at t=1.
!
!       during processing:
!
! p0      The point on the curve at t=0.
! p1      The point on the curve at t=1.
! info > 1  Curve tracing failed.
!      = 0  Found the root at t = 1.

!  .. Local variable

      INTEGER  i, i1, j, icnt, info, icnt_inf, icnt_fl, ntimes1
      LOGICAL  retrace, yn_infnty, yn_real
      REAL     dran2
      DOUBLE PRECISION smallnum
      COMPLEX*16       ctmp

!  .. Executable code ..

      smallnum = 1.0d-8

      retrace = .FALSE.
      yn_real = .TRUE.
      DO j = ilow(1), ihgh(n)
         IF(ABS(DIMAG(ap_scl(j))) .GT. smallnum) THEN
            yn_real = .FALSE.
            GOTO 12
         ENDIF
      ENDDO
 12   IF(yn_real) THEN
         DO i = ilow(1), ihgh(n)
            ap_real(i) = DBLE(ap_scl(i))
         ENDDO
      ENDIF

c     icnt = 1459874
c     icnt = 98743
      OPEN(4,file='SRC/SOLVER/iseed.dat')
          READ(4,*) icnt
      CLOSE(4)
      DO j = 1, n
         ctmp = cmplx(1.0d0-2.0d0*dran2(icnt),1.0d0-2.0d0*dran2(icnt))
         DO i = ilow(j), ihgh(j)
            ap_scl(i) = ctmp*ap_scl(i)
         ENDDO
      ENDDO
      do i = 1, n
         ctmp = cmplx(1.0d0-2.0d0*dran2(icnt),1.0d0-2.0d0*dran2(icnt))
         ap_scl(ihgh(i)) = ap_scl(ihgh(i)) + ctmp*1.0d-10
         rdhz(i,1) = DBLE(ctmp)*1.0d-10
         rdhz(i,2) = DIMAG(ctmp)*1.0d-10
      enddo
c     OPEN(4,file='SRC/SOLVER/iseed.dat')
c         icnt = dran2(icnt)*700000
c         WRITE(4,*) icnt
c     CLOSE(4)

      DO i = ilow(1), ihgh(n)
         cft(i) = aq(i) - ap_scl(i)
      ENDDO

      icnt = 0                             ! count of convergent curves
      icnt_inf = 0                         ! count of infinite curves
      DO j = 1, nrts
         IF(ntimes(j) .EQ. 0) THEN
            IF(debug) THEN
              WRITE(10,*) ' '
              WRITE(10,*) 'For the ', j, 
     &           '-th curve, the initial point at t=0', 
     &                  ' is NOT avaliable'
              WRITE(10,*) '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
            ENDIF
         ELSE
            IF(debug) THEN
              WRITE(10,*) ' '
              WRITE(10,*) 'TRACING THE ', j, 
     &           '-TH CURVE OF THE LINEAR HOMOTOPY:'
            ENDIF

            DO i = 1, n
               p0(i) = roots0(i,j)
            ENDDO

            ntimes1 = ntimes(j)
            CALL zonepth(lda,n,ilow,ihgh,aq,ap_scl,cft,p0,p1,ntimes1,
     &                   yn_infnty,info,
     &                   v_old,v_new,dhz,alpha,h,p2,cf,alpha2)

            IF(info .eq. -1) THEN
               icnt = icnt + 1
               ntimes(j) = -ntimes(j)
               DO i = 1, n
                  roots1(i,j) = p1(i)
               ENDDO
               CALL connum(lda,n,ap_scl,p1,dhz,dhegn(j))
               IF(debug) THEN
                 WRITE(10,*) '   direct'
               ENDIF
            ELSE
               IF(yn_infnty) THEN
                  ntimes(j) = 0
                  IF(debug) THEN
                    icnt_inf = icnt_inf + 1
                    WRITE(10,*) '   Infinite curve'
                  ENDIF
               ELSE
                  ntimes(j) = ntimes(j) + 1
                  IF (debug) THEN
                     WRITE(10,*) '  Tracing failed for first round '
                  ENDIF
               ENDIF
            ENDIF
         ENDIF
      ENDDO

      IF(debug) THEN
        WRITE(10,*) '# of roots found in the first round :  ', icnt
        WRITE(10,*) '# of divigent curves in the first round : ', 
     &             icnt_inf
        WRITE(10,*) '# of curves to be traced in the second round : ', 
     &                                          nrts-icnt-icnt_inf
      ENDIF

      IF (icnt .LT. nrts) retrace = .TRUE.

!     To check curve jumpings

      CALL chck_rtseq(lda,n,roots1,nrts,ntimes,hnrm,dhegn,retrace)

!     Retrace all trouble curves.

      icnt    = 0
      icnt_fl = 0                              ! # of trouble curves
      IF(retrace) THEN                         ! To retrace the trouble curves
         DO j = 1, nrts
            IF(ntimes(j) .GT. 0) THEN

               IF(debug) THEN
                 WRITE(10,*) ' '
                 WRITE(10,*) '2 RETRACE THE ', j, '-TH CURVE:'
               ENDIF

               DO i = 1, n
                  p0(i) = roots0(i,j)
               ENDDO

               ntimes1 = ntimes(j)
               CALL zonepth(lda,n,ilow,ihgh,aq,ap_scl,cft,p0,p1,
     &                      ntimes1,yn_infnty,info,
     &                      v_old,v_new,dhz,alpha,h,p2,cf,alpha2)


               IF(info .eq. -1) THEN
                  icnt = icnt + 1
                  ntimes(j) = -ntimes(j)
                  DO i = 1, n
                     roots1(i,j) = p1(i)
                  ENDDO
                  CALL connum(lda,n,ap_scl,p1,dhz,dhegn(j))
                  IF(debug) THEN
                    WRITE(10,*) '   direct'
                  ENDIF
               ELSE
                  IF(yn_infnty) THEN
                     ntimes(j) = 0
                     IF(debug) THEN
                       icnt_inf = icnt_inf + 1
                       WRITE(10,*) '   Infinite curve'
                     ENDIF
                  ELSE
                     ntimes(j) = 0
                     IF(debug) THEN
                        icnt_fl = icnt_fl + 1
                        WRITE(10,*) '   Tracing failed for 2nd round'
!                       The curves diverge before close to t=1.
!                       End-game can not be applied.
                     ENDIF
                  ENDIF
               ENDIF
            ENDIF
         ENDDO
      ENDIF

      IF(debug) THEN
        WRITE(10,*) '# of roots found in the second round :  ', icnt
        WRITE(10,*) '# of divigent curves in total : ', icnt_inf
        WRITE(10,*) '# of curves diverging before close to t=1 : ',icnt
      ENDIF

!     Retrce only the curves causing curve-jumpings

      DO i = 1, gmax_trace
         retrace = .FALSE.
         CALL chck_rtseq(lda,n,roots1,nrts,ntimes,hnrm,dhegn,retrace)
         IF(.NOT. retrace) GOTO 22
         icnt = 0
         icnt_inf = 0
         icnt_fl = 0
         DO j = 1, nrts
            IF(ntimes(j) .GT. 0) THEN
               IF(debug) THEN
                 WRITE(10,*) ' '
                 WRITE(10,*) i+2, 'RETRACE THE ', j, '-TH CURVE:'
               ENDIF

               DO i1 = 1, n
                  p0(i1) = roots0(i1,j)
               ENDDO

               ntimes1 = ntimes(j)+1
c              ntimes1 = ntimes(j)
               CALL zonepth(lda,n,ilow,ihgh,aq,ap_scl,cft,p0,p1,
     &                      ntimes1,yn_infnty,info,
     &                      v_old,v_new,dhz,alpha,h,p2,cf,alpha2)

               IF(info .eq. -1) THEN
                  icnt = icnt + 1
                  ntimes(j) = -ntimes(j)
                  DO i1 = 1, n
                     roots1(i1,j) = p1(i1)
                  ENDDO
                  CALL connum(lda,n,ap_scl,p1,dhz,dhegn(j))
                  IF(debug) THEN
                    WRITE(10,*) '   direct'
                  ENDIF
               ELSE
                  IF(yn_infnty) THEN
                     ntimes(j) = 0
                     IF(debug) THEN
                       icnt_inf = icnt_inf + 1
                       WRITE(10,*) '   Infinite curve'
                     ENDIF
                  ELSE
                     ntimes(j) = ntimes(j) + 1
                     IF(debug) THEN
                        icnt_fl = icnt_fl + 1
                        WRITE(10,*) '   Tracing failed'
                     ENDIF
                  ENDIF
               ENDIF
            ENDIF
         ENDDO

         IF(debug) THEN
           WRITE(10,*) '# of roots found in ', i+2, '-th round :  ',
     &               icnt
           WRITE(10,*) '# of divigent curves in this round : ', 
     &               icnt_inf
           WRITE(10,*) '# of curves diverging before close to t=1 : ',
     &               icnt
         ENDIF

      ENDDO

 22   DO i = 1, n
         ap_scl(ihgh(i)) = ap_scl(ihgh(i)) 
     &                     - cmplx(rdhz(i,1),rdhz(i,2))
      ENDDO

!     To refine the zeros got.

      CALL rts_refine(lda,n,roots1,nrts,scl,ntimes,
     &                ap_scl,ap_real,ap,yn_real,
     &                rdhz,alpha,alpha2,dhz,h,p0,hnrm,dhegn)

!     To print the results got

      CALL rts_prnt(lda,n,roots1,nrts,ntimes,hnrm,dhegn)

      RETURN
      END
