      SUBROUTINE znwtnat1(lda,n,ap,p0,info,dhz,h)

      IMPLICIT none

      include '../global.h'

!  .. Scalar Arguments ..

      INTEGER lda, info, n

!  .. Array Arguments ..

      COMPLEX*16 ap(*), p0(*)

!  .. Local array variables ..

      COMPLEX*16 h(*), dhz(lda,*)

!  .. PURPOSE ..

!	Newton's iteration on the level t=1

!  .. Input and output ..

! ap     The coefficients of the system
! p0     The points for Newton's iteration
! info= 2 Newton's iteration divergens.
!     = 1 Coefficient matrix dhz is singular.
!     = 0 Newton's iteration converges.
!
! dhz    Jacobian matrix for Newton's method
! h      Function values for Newton's method

!  .. Local Variables ..

      INTEGER          i, iter
      DOUBLE PRECISION nrm_p0, nrm_p1, nrm_dif, rcond, dtmp
      DOUBLE PRECISION dznrm2

!  .. Executable Code ..

      iter = 0

      CALL zgetp(ap,p0,h)
      nrm_p0 = dznrm2(n,h)

      DO WHILE (iter .le. gmax_nwtn) 

!        One step of Newton's iteration.

         CALL zgetdp(lda,ap,p0,dhz)
         CALL cqrdc_slv(lda,n,dhz,h,info)

         rcond = 1.0d0
         DO i = 1, n
            rcond = MIN(rcond, ABS(DBLE(dhz(i,i)))+ABS(DIMAG(dhz(i,i))))
         ENDDO
         IF (nrm_p0 .lt. 1.0d-6 .AND. rcond .lt. 1.0d-8) THEN
            info = 0
            IF(debug) THEN
              WRITE(10,*) '  Norm of h at root : ', nrm_p1
c             WRITE(*,*) '  Norm of h at root : ', nrm_p1
            ENDIF
            RETURN
         ENDIF

         IF(info .eq. 1) RETURN           !! the coefficint matrix is singular
         DO i = 1, n
            p0(i) = p0(i) - h(i)
         ENDDO
         nrm_dif = dznrm2(n,h)

         if (nrm_dif .gt. 1.0d0) then    !Is it OK?
            info = 2
            return
         endif

         CALL zgetp(ap,p0,h)
         nrm_p1 = dznrm2(n,h)

         dtmp = 1.0d0
         DO i = 1, n
            dtmp = MAX(dtmp, ABS(DBLE(p0(i)))+ABS(DIMAG(p0(i))))
         ENDDO
         dtmp = MIN(dtmp,1.0d5)   !! To keep dtmp not too big
         IF (nrm_p1/dtmp .lt. geps_at1 .OR.  
     &       (nrm_p1/dtmp .lt. geps_at1*100 .AND. rcond .lt. 1.0d-4) 
     &       .OR. nrm_dif .lt. geps_at1*100 ) THEN

            IF(debug) THEN
              WRITE(10,*) '  Norm of h at root : ', nrm_p1
            ENDIF

            info = 0
            RETURN
         ENDIF

         IF(nrm_p1/nrm_p0 .ge. grsdl_fctr) THEN
            info = 2
            RETURN                                                !! divergent
         ENDIF

         nrm_p0  = nrm_p1
         iter    = iter + 1
      ENDDO

      info = 2
 
      END                
