      SUBROUTINE zonepth(lda,n,ilow,ihgh,aq,ap,cft,p_old,p_new,ntimes,
     &                   yn_infnty,info,
     &                   v_old,v_new,dhz,alpha,h,p0,cf,alpha2)

      IMPLICIT none

      include '../global.h'

!  .. Scalar Arguments

      INTEGER lda, n, info, ntimes
      LOGICAL yn_infnty

!  .. Array Arguments

      INTEGER          ilow(*), ihgh(*)
      COMPLEX*16       ap(*), aq(*), cft(*), p_old(*), p_new(*)

!  .. Local array variables ..

      COMPLEX*16       v_old(*), v_new(*), dhz(lda,*)
      DOUBLE PRECISION alpha(*)

      COMPLEX*16       h(*), p0(*), cf(*)
      DOUBLE PRECISION alpha2(*)

!  .. PURPOSE ..

!  	zonepth.f is to follow one homotopy curve of a linear homotopy.

!  .. Input and output ..

! p_old, p_new  The last two points known on the curve.
! v_old, v_new  The tangent vectors of the curve at p_old, p_new.
! stp_old  The step size to get p_old from the point before it. 
! stp_new  The step size to get p_new from p_old. 
! t_new    The corresponding t for p_new. 
! stp_c    The current step size to predict the next point.
! info > 0  Curve tracing failed. 
!      = 0  Newton's iteration converges.
!      =-1  Found a root at t=1.
!
! dhz  Jacobian matrix of the homotopy WRT z.
! h    Function values of the homotopy.
! p0   The points generated by Newton's method.

!  .. Local variables ..
 
      INTEGER          i, nstp
      DOUBLE PRECISION t_new, stp_old, stp_new, stp_c
      DOUBLE PRECISION stp_init, stp_max
      LOGICAL          yn_alpha

      stp_init  = gstp_init/(10.0d0**(ntimes-1))
      stp_max   = gstp_max/(10.0d0**(ntimes-1))
      yn_alpha  = .FALSE.
      yn_infnty = .FALSE.

      nstp = 0              ! # of steps. For testing only.

!     To get the new tangent vector at initial point p_old

      CALL zgetdp(lda,aq,p_old,dhz)
      CALL zgetp(cft,p_old,v_old)

      CALL cqrdc_slv(lda,n,dhz,v_old,info)
      IF(info .ne. 0) THEN
c     IF(info .eq. 1) THEN
         IF(debug) THEN
           WRITE(10,*) '  The dhz is singular at t= 0. Impossible.'
           WRITE(10,*) '  !!!!!!!!!!!!!!!!!!!!!!!!!'
         ENDIF
         RETURN
      ENDIF
       
!     To go out one step

      stp_c = stp_init
      CALL z1ststp(lda,n,ilow,ihgh,aq,ap,cft,p_old,v_old,
     &             stp_c,p_new,v_new,info,
     &             dhz,h,cf)

      nstp = nstp + 1

      IF(info .GT. 0) THEN
         IF(debug) THEN
           DO i = 1, n
             WRITE(10,*) '  ', p_new(i)
           ENDDO
           WRITE(10,*) '  Failed at z1ststp1.f with info = ', info
           WRITE(10,*) '  The # of times traced : ', ntimes
           WRITE(10,*) '  The # of steps  is : ',nstp
           WRITE(10,*) '  The current t =  ', t_new
         ENDIF

         RETURN
      ENDIF

      t_new   = stp_c
      stp_old = stp_c
      stp_new = stp_c
!     stp_c   = stp_c

!     To go out one more step

 20   CALL z1step(lda,n,ilow,ihgh,aq,ap,cft,p_old,v_old,
     &            p_new,v_new,t_new,stp_new,stp_c,ntimes,
     &            info,alpha,yn_alpha,yn_infnty,
     &            dhz,h,p0,cf,alpha2)

      nstp = nstp + 1

      IF(info .GT. 0) THEN
         IF(debug) THEN
           WRITE(10,*) '  Failed at z1step1.f with info = ', info
           WRITE(10,*) '  The # of times traced : ', ntimes
           WRITE(10,*) '  The # of steps  is : ',nstp
           WRITE(10,*) '  The current t =  ', t_new
           DO i = 1, n
              WRITE(10,*) '  ', p_new(i)
           ENDDO
         ENDIF

         RETURN
      ELSE
         IF(info .EQ. -1) THEN                    ! p_new is the solution at t=1
            IF(debug) THEN
              WRITE(10,*) '  The root is located at the level t=1'
              WRITE(10,*) '  The # of times traces : ', ntimes
              WRITE(10,*) '  The # of steps  is : ',nstp
            ENDIF

            RETURN
         ENDIF
      ENDIF

!     To decide next step size	

      IF(stp_c .ge. stp_new .AND. stp_new .ge. stp_old) THEN
         stp_old = stp_new
         stp_new = stp_c
         stp_c   = 2*stp_c
      ELSE
         stp_old = stp_new
         stp_new = stp_c
!        stp_c   = stp_c
      ENDIF

      IF(t_new .gt. 0.999) THEN
         stp_c = MIN(stp_c,stp_max*0.1)
      ELSE
         stp_c = MIN(stp_c,stp_max)
      ENDIF
      GOTO 20

      END                
