      SUBROUTINE dgauss(lda,n,a,v,info)
 
      IMPLICIT none
 
!  .. Scalar variables ..
 
      INTEGER lda, n, info
 
!  .. Array variables ..
 
      DOUBLE PRECISION a(lda,*), v(*)
 
!  .. Purpose ..
 
!     this subroutine is to use gaussian elimination method to get 
!     the normal vector v(*) with v(1)=1. The matrix is stored in
!     the form (b,A) for the system (b,A)(1,alpha)^T=0.
 
!     output: a is changed
 
!  .. Local scalar variables ..
 
      INTEGER          itmp1 
      DOUBLE PRECISION tmp2, eps
      INTEGER          i, j, k
 
!  .. Executable coed ..
 
      eps  = 1.0d-13
      info = 1
 
      do i = 2, n
!        to find the first non-zero elemnet in i-th column for Gauss
         itmp1 = i-2
         do j = i-1,n
            if(abs(a(j,i)) .ge. eps) then
               itmp1 = j
!              itmp1-th row has the first nonzero element
               goto 10
            endif
         enddo

         if (itmp1 .eq. i-2 ) then
            info=-1
!           the matrix a is not of full rank
            return
         endif

!        exchange the two rows. make the (i,i) element nonzero
 10      if (itmp1 .gt. i-1) then
            tmp2=a(itmp1,1)
            a(itmp1,1)=a(i-1,1)
            a(i-1,1)=tmp2
            do j=i, n+1
               tmp2=a(itmp1,j)
               a(itmp1,j)=a(i-1,j)
               a(i-1,j)=tmp2
            enddo
         endif
!
!        make all other elements in the column zero
         do j=i, n
            if(ABS(a(j,i)) .ge. eps) then
               a(j,1)=a(j,1)-a(i-1,1)*a(j,i)/a(i-1,i)
               do k=i+1, n+1
                  a(j,k)=a(j,k)-a(i-1,k)*a(j,i)/a(i-1,i)
               enddo
!              a(j,i)=0.0d0
            endif
         enddo
      enddo 
!
      if (ABS(a(n,n+1)) .le. eps) then
         info=-1
!        the matrix a is not of full rank
         return
      endif

!     Ready to get the normal vector.
        
      v(1) = 1.0d0
      v(n+1) = -a(n,1)/a(n,n+1)
      do i = n, 2, -1
         v(i) = 0.0d0
         DO j = i+1, n+1
            v(i) = v(i) + a(i-1,j)*v(j)
         ENDDO
         v(i)=(-v(i)-a(i-1,1))/a(i-1,i)
      enddo 
!
      return
      end
               
        
