      SUBROUTINE dslvv(lda,n,aA)
 
      IMPLICIT none

!  .. Scalar Variables ..
 
      integer lda, n
 
!  .. Array variables ..

      DOUBLE PRECISION aA(lda,*)
 
!  .. Purpose ..
 
! This program is about solving a linear system aA[1:n] * v = x symbolically, 
! where we know that aA[1:n] is nonsingular already. 
! So we do partial-pivoting exactly n times.
! But here x is a vector composed of variables, i.e. we want to write
! our formal solution v in terms of variables in x. For instance,
! [ 1 2 3    [v1    [x1
!   4 5 6   * v2  =  x2
!   7 8 9 ]   v3]    x3]
! Then we can do the Gauss elimination to [1 2 3 -1  0  0
!                                          4 5 6  0 -1  0
!                                          7 8 9  0  0 -1]
! initial aA=[A:-I] where A is the coefficient matrix of the equation.
! final form aA=[I:V] where V(x1,...xn)^T is a solution.
 
!  .. Local Variables
 
      INTEGER          ind, i, j, k, i2
      DOUBLE PRECISION pivot, dtmp, smallnum
 
!  .. Executable Code
 
      smallnum = 1.0d-11
 
!     Upper-diagonalize
 
      do j = 1, n               ! 1 to 2*n is also OK
        pivot = smallnum
        ind = 0
        do i = j, n
          dtmp = ABS(aA(i,j))
          if (dtmp .gt. pivot) then
            pivot = dtmp
            ind = i
          endif
        enddo
!       Switch rows :
        if (ind .ne. j) then
          do k = 1, 2*n
            dtmp      = aA(ind,k)
            aA(ind,k) = aA(j,k)
            aA(j,k)   = dtmp
          enddo
        endif

        do i= j+1, n
          dtmp= aA(i,j)/aA(j,j)
          DO i2 = j+1, 2*n
             aA(i,i2) = aA(i,i2) - dtmp*aA(j,i2)
          ENDDO
          aA(i,j) = 0.0d0
        enddo
      enddo

!     BACK-SUBSTITUTION

      DO j = n+1, 2*n
         DO i = 1, n
            aA(i,j) = -aA(i,j)
         ENDDO
      ENDDO
      do i = n, 1, -1
        DO j = n+1, 2*n
           aA(i,j) = aA(i,j)/aA(i,i)
        ENDDO
        aA(i,i) = 1.0d0
        do i2 = i-1, 1, -1
          DO j = n+1, 2*n
             aA(i2,j) = aA(i2,j) - aA(i2,i)*aA(i,j)
          ENDDO
          aA(i2,i) = 0.0d0
        enddo
      enddo

      return
      end

