      SUBROUTINE lftng_un(n,ia,lft,ilow,ihgh,ns,irep,ptscmb,ncmb,
     &                    lda,ldb,
     &                    noe,nos,ja,ib,jtmp1,jtmp2,
     &                    a,b,binv,v,vn,s)
 
 
      IMPLICIT NONE
 
!  .. Scalar Arguments ..
 
      INTEGER  lda, ldb, n, ns, ncmb
 
!  .. Array Arguments ..
 
      INTEGER ia(lda,*), ilow(*), ihgh(*), irep(*), ptscmb(*)
      DOUBLE PRECISION lft(*)
 
!  .. Local array ..

      INTEGER noe(*), nos(*), ja(*), ib(*), jtmp1(0:*), jtmp2(0:*)
      DOUBLE PRECISION a(*), b(*), binv(ldb,*), v(*), vn(lda,*), s(*)
 
!  n      the dimension of the poly system
!  ncmb   # of type (1,...,1) cells
!
!  ia     the support as usual
!  lft    (input)  the original lifting values (lft(2) to lft(tnop+1); 
!         (output) the optimal solution. 
!                  M=lft(1). lft(2),... lft(tnop+1) are the lifting values of
!                  the support
!  ilow,ihgh   points of i-th support are placed from ilow(i)-th column to 
!              ihgh(i)-th in ia
!  ptscmb records the indices of pts of the (1,...,1) type cells in ia
!
!  a      nonzero elements of the coefficent matrix
!         which is stored in sparse form (determined by noe, nos, ia)
!  ja     ja(i) is the column index of a(i) in matrix form of a 
!  noe    noe(i) is the # of nonzero elements in i-th row of a
!  nos    nos(i) is the position in a of the first nonzero element in 
!         the i-th row of a
 
!  v      work space.
!  vn     n by 2*n work space for recording the sol of the n eqns for an
!         inner normal
!  b      the right-side constant terms of the linear programming problem
!  ib     work space for linear programming.
!  binv   work space for the linear programming problem

!  .. Local Variables
 
      INTEGER  nvtx, nlst, npts, neqs
      INTEGER  na, ma, i, i0, i1, i2, icl, j, info
      INTEGER  ice, icnt, icntr, ma_
 
      DOUBLE PRECISION dtmp, dmaxp, dminp, dma, dmi, dm, smallnum, d1
      PARAMETER (d1 = 1.0d0)           !! the minimum of the t-powers
 
!  .. Executable Code
 
      smallnum = 1.0d-10 
      nvtx = ns
      DO i = 1, ns
         nvtx = nvtx + irep(i)      !!# of vertices in a facet
      ENDDO
      na   = ihgh(ns) - nvtx          !!# of variables for the LP
 
!     (1) get the inner normal of the first cell
 
      npts = 1
      neqs = 0
      DO i = 1, ns
         DO i1 = 1, irep(i)
            neqs         = neqs + 1
            binv(neqs,1) = lft(ptscmb(npts)) - lft(ptscmb(npts+i1))
            DO j = 1, n
               binv(neqs,j+1) = ia(j,ptscmb(npts)) - 
     &                          ia(j,ptscmb(npts+i1))
            ENDDO
         ENDDO
         npts = npts + irep(i) + 1
      ENDDO

      CALL dgauss(ldb,n,binv,v,info)                            !!v=(1,*)
 
!     (2) make the U change and move the lowest to 0
 
      npts = 1
      DO i = 1, ns
         dtmp = lft(ptscmb(npts))
         DO i1 = 1, n
            dtmp = dtmp + v(i1+1)*ia(i1,ptscmb(npts))
         ENDDO
         DO j = ilow(i), ihgh(i)
            lft(j) = lft(j) - dtmp
            DO i1 = 1, n
               lft(j) = lft(j) + v(i1+1)*ia(i1,j)
            ENDDO
         ENDDO
         npts = npts + irep(i) + 1
      ENDDO
 
!     now, to delete the lft's of the 1st cell to fit the LP
 
      npts = 1
      DO i = 1, nvtx - 1
        DO i1 = ptscmb(i)+1, ptscmb(i+1)-1
           lft(i1-npts) = lft(i1)
        ENDDO
        npts = npts + 1
      ENDDO
      DO i = 0, ihgh(ns)-(ptscmb(nvtx)+1)
         lft(ptscmb(nvtx)-npts+1+i) = lft(ptscmb(nvtx)+1+i)
      ENDDO
       
      ma = 0
 
!     To form the part of the matrix from the first cell
 
      DO i = 1, na
         a(i) = 1.0d0
      ENDDO
      DO i = 1, na
         noe(i) = 1
      ENDDO
      DO i = 1, na
         ja(i)  = i
         nos(i) = i
      ENDDO
      ma  = na                     !! index of last column stored in a so far
      ice = na                     !! ice : # of elements so far in a
 
!     To get the min and max from the first cell
 
      dma = -1.0d150
      DO i = 2, na
         dma = MAX(dma,lft(i))
      ENDDO
      dmi = 1.0d150
      DO i = 2, na
         dmi = MIN(dmi,lft(i))
      ENDDO
      dm  = d1*dma/dmi
 
!     continue forming the matrix
 
      DO icl = 1, ncmb-1             !!for each cell starting from the second
 
         nlst = nvtx*icl
 
         npts = 1
         neqs = 0
         DO i = 1, ns                                                !! form A
            DO i1 = 1, irep(i)
               neqs = neqs + 1
               DO j = 1, n
                  vn(neqs,j) = ia(j,ptscmb(nlst+npts))                     
     &                        - ia(j,ptscmb(nlst+npts+i1)) 
               ENDDO
            ENDDO
            npts = npts + irep(i) + 1
         ENDDO                          !!coefficient matrix of the equations
         DO j = n+1, 2*n
            DO i = 1, n
               vn(i,j) = 0.0d0
            ENDDO
         ENDDO
         DO j = 1, n
            vn(j,n+j) = -1.0d0
         ENDDO
 
         CALL dslvv(lda,n,vn)       !!vn(:,n+1:2n) is inverse of vn(:,1:n)
 
         ma_  = 0
         icnt = 1
         DO i = 1, ns
           jtmp1(0)           = ilow(i)
           DO i1 = 0, irep(i)
              jtmp1(i1+1) = ptscmb(nlst+icnt+i1) + 1
           ENDDO
           DO i1 = 0, irep(i)
              jtmp2(i1) = ptscmb(nlst+icnt+i1) - 1
           ENDDO
           jtmp2(irep(i)+1)   = ihgh(i)
           DO i0 = 0, irep(i)+1 
             DO j = jtmp1(i0), jtmp2(i0)    !! for each point c, form e_c
               ma_ = ma_ + 1
               DO i1 = 1, n                 !! v=(c-a_i)A^{-1}
                  v(i1) = 0.0d0
                  DO i2 = 1, n
                     v(i1) = v(i1) + 
     &                 (ia(i2,j)-ia(i2,ptscmb(nlst+icnt)))*vn(i2,n+i1)
                  ENDDO
               ENDDO
               DO i1 = 1, ihgh(ns)
                  binv(i1, ma_) = 0.0d0
               ENDDO
               npts = 1
               neqs = 0
               DO i1 = 1, ns
                 binv(ptscmb(nlst+npts),ma_) = 0.0d0
                 DO i2 = 1, irep(i1)
                    binv(ptscmb(nlst+npts),ma_) = 
     &                binv(ptscmb(nlst+npts),ma_) - v(neqs+i2)
                 ENDDO
                 DO i2 = 1, irep(i1)
                   binv(ptscmb(nlst+npts+i2),ma_) = v(neqs+i2)
                 ENDDO
                 npts = npts + irep(i1) + 1
                 neqs = neqs + irep(i1)
               ENDDO
               binv(j,ma_) = 1.0d0
               binv(ptscmb(nlst+icnt),ma_) =            
     &                     binv(ptscmb(nlst+icnt),ma_) - 1.0d0
             ENDDO
           ENDDO
           icnt = icnt + irep(i) + 1
         ENDDO
 
!        After done the part of the matrix for a cell, 
!        delete the columns WRT the pts in first cell
 
         npts = 1
         DO i1 = 1, nvtx
            jtmp1(1) = ptscmb(i1) + 1
            IF(i1 .LT. nvtx) THEN
               jtmp2(1) = ptscmb(i1+1) -1
            ELSE
               jtmp2(1) = ihgh(ns)
            ENDIF
            DO i2 = jtmp1(1), jtmp2(1)
               DO j = 1, ma_
                  binv(i2-npts,j) = binv(i2,j)
               ENDDO
            ENDDO
            npts = npts + 1
         ENDDO
 
!        To get the min, max for each inequality
 
         dmaxp = -1.0d0
         dminp = 1.0d150     !! HUGE(1.0_wp)
         DO j = 1, ma_
            dtmp = 0.0d0
            DO i = 2, na
               dtmp = dtmp + binv(i,j)*lft(i)
            ENDDO
            if(dtmp .LT. 0.0d0) then                             !!for testing
               write(10,*) 'wrong, the power of t is ', dtmp
            endif
            IF(dtmp .GT. dmaxp) dmaxp = dtmp
            IF(dtmp .LT. dminp) dminp = dtmp
         ENDDO
         dma= max(dma,dmaxp)
         dmi= min(dmi,dminp)
         dm = max(d1*dmaxp/dminp,dm) !max of powers after scaling so far
 
!        To delete the repeated points
 
         icntr = 0        !! count # of columns undeleted from this cell
         DO j = 1, ma_    !! for each new column
            icnt = 0      !! count # of nonzero elements
            DO i = 2, na
               IF(ABS(binv(i,j)) .GT. smallnum) THEN
                  icnt         = icnt + 1
                  a(ice+icnt)  = binv(i,j)
                  ja(ice+icnt) = i
               ENDIF
            ENDDO
!           To compare with the inequalities got before
            DO i = 1, ma
               IF(noe(i) .EQ. icnt) THEN  !!#s of nonzero elements equals
                  DO i1 = 0, icnt-1
                     IF(ja(nos(i)+i1) .EQ. ja(ice+1+i1)) THEN
                        IF (abs(a(nos(i)+i1)-a(ice+1+i1)) 
     &                      .GT. smallnum) GOTO 23
                     ELSE
                        GOTO 23
                     ENDIF
                  ENDDO
                  GOTO 25                                !! equals. skip.
               ENDIF
 23         ENDDO
 
!           Not equal. To accept the new inequality
 
            icntr = icntr + 1
            noe(ma+icntr) = icnt
            nos(ma+icntr) = ice + 1
            ice = ice + icnt
 
 25      ENDDO
         ma = ma + icntr
 
      ENDDO
 
!     ice number of elements are in a so far
!     To write 1< * <M into two inequalities in form of a^Tx < b
 
      DO j = 1, ma
         noe(ma+j) = noe(j) + 1
         nos(ma+j) = ice + 1
         ja(ice+1) = 1                       !! for M in * < M
         DO i = 0, noe(j)-1
            ja((ice+2)+i) = ja(nos(j)+i)
         ENDDO
         a(ice+1) = -1.0d0                  !! for M in * < M
         DO i = 0, noe(j)-1
            a((ice+2)+i) = a(nos(j)+i)
         ENDDO
         DO i = 0, noe(j)-1
            a(nos(j)+i) = -a(nos(j)+i)
         ENDDO
         b(j)     = -d1
         b(ma+j) = 0.0d0
         ice = ice + noe(ma+j)
      ENDDO
 
      ma = 2*ma
 
      DO j = 1, na
         DO i = 1, na
            binv(i,j) = 0.0d0
         ENDDO
      ENDDO
      DO j = 1, na
         binv(j,j) = 1.0d0
      ENDDO
      DO i = 1, na
         ib(i) = 0
      ENDDO
      lft(1) = dma*(d1+1)/dmi
      DO i = 2, na
         lft(i) = lft(i)*((d1+1.0d0)/dmi)
      ENDDO
 
      WRITE(10,*) 'before balancing:' 
      WRITE(10,*) 'smallest power of t:', dmi
      WRITE(10,*) 'largest power of t:', dma
      WRITE(10,*) 'largest power of t after scaling:', dm
      WRITE(10,*) ' '
      WRITE(10,*) '# of variables:', na
      WRITE(10,*) '# of different constraints:', ma
      WRITE(10,*) '# of total constraints (may equal):', ice
 
      CALL alg2_sp(a,ma,na,nos,noe,ja,b,lft,binv,ib,info,ldb,s)
 
      write(10,*) 'largest power of t after optimization: ', lft(1)
 
!     To change the lft into the original order
!     and make the lifting values of the first cell be zero.
 
      DO i = ihgh(ns), ptscmb(nvtx)+1, -1
         lft(i) = lft(i-nvtx)
      ENDDO
      lft(ptscmb(nvtx)) = 0.0d0
      DO i = nvtx-1, 1, - 1
         DO i1 = ptscmb(i+1)-1, ptscmb(i)+1, -1
            lft(i1) = lft(i1-i)
         ENDDO
         lft(ptscmb(i)) = 0.0d0
      ENDDO

      RETURN
      END
