      subroutine dlp1_1pts(lda,na,a,b,c,lpindx,nlpindx,p1,
     &                    nuptsin,nnuptsin,
     &                    jj,x,dinv,
     &                    x_s,jj_s,dinv_s)
 
      implicit none
 
*  .. Purpose:

*     This is to solve 
*            min c^T x
*            a^T_i x < b_i, i=lpindx(j), j=1,...,nlpindx
*     x, jj, and dinv are available.
*     During the computation, it records all points (>=p1) which pass 
*     the 1-point test AND x, jj, dinv for initial point of next round
*     computation.

*  .. Scalar Arguments ..
      integer          lda, na, nlpindx, p1, nnuptsin
 
*  .. Array Arguments ..
      integer          jj(*), lpindx(*), nuptsin(*) 
      double precision b(*), c(*), x(*)
      double precision a(lda,*), dinv(lda,*)
      integer          jj_s(lda,*)
      double precision x_s(lda,*), dinv_s(lda,*)
 
*  .. Input

*        lda: leading dimension of a, >= na+1
*          a: each column is the coefficients of a constraint
*             its active part is a(1:na,1:1:lpindx(nlpindx))
*          b: constant terms, b(1:1:lpindx(nlpindx)) is active
*          x: a nondegenerate feasible solution
*          c: coefficients of the cost function
*         jj: the indices of the base constraints
*       dinv: the inverse of a base matrix whose rows are the coefficients
*             of the base constraints
*    nuptsin: nuptsin(1:nnuptsin) are the points passed 1-point test so far
*   nnuptsin: # of points passed 1-point test so far

*  .. Output

*          x: a nondegenerate feasible solution
*         jj: the indices of the base constraints
*       dinv: the inverse of a base matrix whose rows are the coefficients
*             of the base constraints
*    nuptsin: nuptsin(1:nnuptsin) are points passed 1-point test so far
*   nnuptsin: # of points passed 1-point test so far
*   x_s,jj_s,dinv_s: saved info for the starting point of next LP problem


*  .. Local workspace
      integer          i, i1, j, k, ell, flag
      double precision eps, vmax, top, bot, sigj, dtmp

*  .. Executable code

      eps  = 1.0d-6

***** step 1                          To find which constraint being kicked out
 
 40   vmax = -1.0d150 
      do i = 1, na
         dtmp = 0.0d0
         do j = 1, na
            dtmp = dtmp + c(j)*dinv(j,i)
         enddo
         if (dtmp .gt. vmax) then
            vmax = dtmp
            k = i
         endif
      enddo
 
      if(vmax .lt. eps) then
         return                                  ! out with an optimal solution
      endif

***** step 2                                 To find which constraint coming in

      sigj = 1.0d150
      ell = 0
      do i1 = 1, nlpindx
         i = lpindx(i1)
         do j = 1, na
            if(jj(j) .eq. i) goto 110
         enddo
         bot = 0.0d0
         do j = 1, na
            bot = bot + a(j,i)*dinv(j,k)
         enddo
         if(bot .ge. -eps) goto 110
         top = -b(i)
         do j = 1, na
            top = top + a(j,i)*x(j)
         enddo
         top = top/bot
         if(top .ge. sigj) goto 110
         sigj = top
         ell = i
 110  enddo
      if(ell .eq. 0) then
         print*, 'unbounded, wrong in dlp1_1pts.f'
         stop                                        ! won't happen in our case
      endif

***** Record the point passed 1-point test

      flag = 0
      if (ell .ge. p1 .and. nuptsin(ell) .eq. 0) then
         nnuptsin = nnuptsin + 1
         nuptsin(ell) = nnuptsin
         flag = 1
      endif

***** step 3                                              To update x, jj, dinv

      do i = 1, na
         x(i) = x(i) - sigj*dinv(i,k)
      enddo

      top = 0.0d0
      do i = 1, na
         top = top + a(i,ell)*dinv(i,k)
      enddo
      if(abs(top) .lt. eps) then
         write(*,*) 'Base matrix is singular in dlp1_1pt.f, WRONG'
         stop                                        ! won't happen in our case
      endif

      top = 1.0d0/top
      do i = 1, na
         dinv(i,k) = dinv(i,k)*top
      enddo
      do j = 1, na
         if(j .ne. k) then
            top = 0.0d0
            do i = 1, na
               top = top + a(i,ell)*dinv(i,j) 
            enddo
            do i = 1, na
               dinv(i,j) = dinv(i,j) - top*dinv(i,k)
            enddo
         endif
      enddo

      jj(k) = ell

      if (flag .eq. 1) then
         do i = 1, na
            x_s(i,nnuptsin) = x(i)
            jj_s(i,nnuptsin) = jj(i)
         enddo
         do j = 1, na
            do i = 1, na
               dinv_s(i,j+(nnuptsin-1)*na) = dinv(i,j)
            enddo
         enddo
      endif

      goto 40

      end
