      subroutine dlp2_1pt_s(lda,ldb,na,a,b,c,lpindx,nlpindx,p1,p2,
     &                      pairs,
     &                      jj,x,dinv,v,
     &                      n_s,x_s,jj_s,dinv_s)

*     This program is to solve
*                    min c^Tx
*                 a_i x <= b_i, for all i with i=lpindx(j), j=1,...,nlpindx
*     with initial x, jj, and dinv given, but jj(i) may be zero.
*     The number of nonzero elements in jj MUST be the rank of the coefficient
*     matrix A. The rank of A may not be full.
*     During the computation, it records all points (>=p1) which pass
*     the 1-point test in pairs and saves jj, x, dinv.
 
      implicit none
 
*  .. Scalar Arguments ..
      integer          lda, ldb, na, nlpindx, p1, p2
 
*  .. Array Arguments ..
      integer          jj(*), lpindx(*),  pairs(ldb,*)
      double precision a(lda,*), b(*), c(*), x(*), dinv(lda,*), v(*)
      integer          n_s, jj_s(lda,*)
      double precision x_s(lda,*), dinv_s(lda,*)

*  .. Input

*        lda: leading dimension of a, >= na+1
*          a: each column is the coefficients of a constraint
*             its active part is a(1:na,1:1:lpindx(nlpindx))
*          b: constant terms, b(1:1:lpindx(nlpindx)) is active
*         p1: the index of the point fixed
*         p2: the index of the point testing
*          x: a nondegenerate feasible solution
*         jj: the indices of the base constraints
*       dinv: the inverse of a base matrix whose rows are the coefficients
*             of the base constraints
*      pairs: 


*  .. Output

*          x: a nondegenerate feasible solution
*         jj: the indices of the base constraints
*       dinv: the inverse of a base matrix whose rows are the coefficients
*             of the base constraints
*      pairs:

 
*  .. Workspace
      integer          ell, k, i, i1, j, flag
      double precision eps, sigj, vmax, bot, top, dtmp

*  .. Executable
 
      eps = 1.0d-6

***** step 1                          To find which constraint being kicked out
 
 40   vmax = -1.0d150
      do i = 1, na
         if (jj(i) .ne. 0) then
            dtmp = 0.0d0
            do j = 1, na
               dtmp = dtmp + c(j)*dinv(j,i)
            enddo
            if (dtmp .gt. vmax) then
               vmax = dtmp
               k = i
            endif
         endif
      enddo

      if(vmax .lt. eps) then
         return                                     ! with an optimal solution
      endif

      do i = 1, na
         v(i) = dinv(i,k)
      enddo
 
***** step 2                                To find which constraint coming in
 
 100  sigj = 1.0d150  

      ell = 0
      do i1 = 1, nlpindx
         i = lpindx(i1)
         do j = 1, na
            if (jj(j) .eq. i) goto 110
         enddo
         bot = 0.0d0
         do j = 1, na
            bot = bot + a(j,i)*v(j)      
         enddo
         if (bot .ge. -eps) goto 110
         top = -b(i)
         do j = 1, na
            top = top + a(j,i)*x(j)
         enddo
         top = top/bot
         if (top .ge. sigj) goto 110
         sigj = top
         ell = i
 110  enddo

      if (ell .eq. 0) then
         print*, 'ell=0, wrong in dlp2_1pt_s.f'
         stop                             ! unbounded, never happen in our case
      endif

***** Record the possible edges after introducing ell-th column in

      flag = 0
      if (ell .ge. p2 .and. pairs(ell,p1) .eq. 0) then
         flag = 1
         pairs(ell,p1) = n_s + 1
         pairs(p1,ell) = n_s + 1
      endif

***** step 3                                              To update x, jj, dinv
 
      do i = 1, na
         x(i) = x(i) - sigj*v(i)
      enddo
      do i = 1, na
         v(i) = a(i,ell)
      enddo
 
*     to find the new B^{-1}
 
      top = 0.0d0
      do i = 1, na
         top = top + v(i)*dinv(i,k)
      enddo
      if (abs(top) .lt. eps) then
         write(*,*) 'singular in dlp2.f. sum = ', top
         stop
      endif
 
      top = 1.0d0/top
      do i = 1, na
         dinv(i,k) = dinv(i,k)*top
      enddo
      do j = 1, na
         if (j.ne.k) then
            top = 0.0d0
            do i = 1, na
               top = top + v(i)*dinv(i,j)
            enddo
            do i = 1, na
               dinv(i,j) = dinv(i,j) - top*dinv(i,k)
            enddo
         endif
      enddo
 
      jj(k) = ell

*     To save jj, x, dinv

      if (flag .eq. 1) then
         n_s = n_s + 1
         do i = 1, na
            x_s(i,n_s) = x(i)
            jj_s(i,n_s) = jj(i)
         enddo
         do j = 1, na
            do i = 1, na
               dinv_s(i,j+(n_s-1)*na) = dinv(i,j)
            enddo
         enddo
      endif

      goto 40

      end
