      SUBROUTINE dmxcll(lda,ldb,n,ns,ia,irep,jlow,jhgh,lft,
     &                  ptscmb,ncmb,
     &                  jcmbs,jlowc,
     &                  j12,jpts,jlnupts,
     &                  ptscmb1,pairs,ptsin,nptsin,lpindx,
     &                  ib,a,b,c,binv,x,v,
     &                  ith2is,ith2na,npn,
     &                  jlowp,jhghp,
     &                  ib_s,x_s,binv_s,jpter_s)

!     This is to find all mixed cells of a general polynomial system
!     (fully mixed, semi-mixed, or fully un-mixed) based on the paper
!     "Mixed volume computation for semi-mixed systems" by Tangan Gao and
!     T.Y. Li.

*     lda >= n+2
*     ldb >= jhgh(ns)+2
*     ia(1:n,1:jhgh(ns))
*     irep(1:n), jlow(1:n), jhgh(1:n)
*     lft(1:jhgh(ns))
*     jcmbs(1:n+ns-1), jlowc(1:n+ns-1), j12(1:n+ns-1), jpts(0:n+ns-1),
*     nptsin(1:n+ns-1), ith2is(1:n+ns-1), ith2na(1:n+ns-1), npn(1:n+ns-1)
*     jlnupts(1:jhgh(ns)),
*     jlowp(1:n,1:n+ns-1), jhghp(1:n,1:n+ns-1)
*     pairs(1:jhgh(ns),1:jhgh(ns)), ptsin(1:jhgh(ns),1:n+ns-1),
*     lpindx(1:jhgh(ns),1:n+ns-1)
*     jlow_s(1:n+ns-1), ib_s(lda,large), x_s(lda,large), binv_s(lda,large*lda),
*     jpter_s(large): large > # of subcells waiting for extension!

      IMPLICIT NONE

!  .. Input and Outpu Variables

!  .. Scalar variables ..
 
      integer lda, ldb, n, ns, ncmb
 
!  .. Array variables ..
 
      integer ia(lda,*), irep(*), jlow(*), jhgh(*), ptscmb(*)
      double precision lft(*)
 
!     n   : # of variables of the polynomial system
!     ns  : # of different support of the system
! jlow, jhgh : ia(1:n,jlow(i):jhgh(i)) stores the pts of i-th support
!     ia  : ia(1:n,1:jhgh(ns)) stores the support of the system
!   irep  : irep(i) is the # of times the support S_i repeats
!     lft : the lifting values of the support

!     ptscmb : contains the mixed cells of right type
!     ncmb : # of the mixed cells of right type
 
!  .. Local scalar variables ..
 
      INTEGER          i, i1, i2, j, j1, kout, nnuptsin, p0, p1, jh,
     &                 flag, ith, ith_, na, nac, ne, is, is_, is_nx,
     &                 jlindx, jhindx, mptinc, info, k, ell, j_s
      DOUBLE PRECISION smallnum, dtmp
 
!  .. Local array variables ..
 
      INTEGER jcmbs(*), jlowc(*), j12(*), jpts(0:*), jlnupts(*),
     &        pairs(ldb,*),ptsin(ldb,*),nptsin(*),lpindx(ldb,*),
     &        ith2is(*), ith2na(*), npn(*), 
     &        jlowp(lda,*), jhghp(lda,*), ptscmb1(*), ib(*),
     &        ib_s(lda,*), jpter_s(*), jlow_s(300)
      DOUBLE PRECISION a(lda,*), b(*), binv(lda,*), x(*), c(*), v(*)
      DOUBLE PRECISION x_s(lda,*), binv_s(lda,*)

!  .. Executable code ..

      smallnum = 1.0d-11
 
      jh = jhgh(ns)
      mptinc = 0
      do i = 1, ns
         mptinc = mptinc + irep(i)
      enddo
      mptinc = mptinc + ns                       ! # of points in a mixed cell

      do i = 1, irep(1)
         ith2is(i) = 1                 ! tells j12(i) belongs to which support
         npn(i) = irep(1) - i       ! # of points needed in S_i for right type
         ith2na(i) = n - i        ! # of variables of LP for the round ith = i
      enddo
      ith = irep(1)
      do i = 2, ns
         ith = ith + 1                   ! the first round 1-point test in S_i
         ith2is(ith) = i
         npn(ith) = irep(i)                              ! since fixed 1 point
         ith2na(ith) = ith2na(ith-1)                 ! since eliminated alpha0
         do i1 = 1, irep(i)
            ith = ith + 1
            ith2is(ith) = i
            npn(ith) = npn(ith-1) - 1
            ith2na(ith) = ith2na(ith-1) - 1
         enddo
      enddo
      ith2is(ith+1) = n+1

      call dpairs(lda,ldb,n,ns,ia,jlow,jhgh,lft,
     &            pairs,
     &            lpindx(1,1),
     &            ib,a,b,c,binv,x,v,
     &            x_s,ib_s,binv_s)

c     print*, 'done relation table .........'
      jlowc(1) = 0 
      jlow_s(1) = 0 
      ne = 0                                 ! to extract the lower edges of S1
      do j = jlow(1), jhgh(1)-1
         do j1 = j+1, jhgh(1)
            if (pairs(j,j1) .gt. 0) then
               ptscmb1(2*ne+1) = j
               ptscmb1(2*ne+2) = j1
               ne = ne + 1
               jpter_s(ne) = pairs(j,j1)
            endif
         enddo
      enddo

c     print*, '# of lower edges in S_1 :', ne
      jcmbs(1) = ne                                      ! # of edges for ith=1
      jlowc(2) = 2*ne       ! jlowc(2)+1 is the place of the first pt for ith=2
      jlow_s(2) = ne     ! jlow_s(2)+1 is the 1st index of saved info for ith=2

      ncmb = 0
      j12(1) = 1 !!j12(1:?) records the indices of the current comb.
                 !!j12(1) represents an edges, other j12(*) a point.
 
 100  jpts(0) =  ptscmb1(2*(j12(1)-1)+1)                 ! Start from the edges
      jpts(1) =  ptscmb1(2*(j12(1)-1)+2)

      if (jhgh(1)-jpts(1) .lt. npn(1)) then  ! cannot be extended to a facet
         if (j12(1) .eq. jcmbs(1)) then
            return                                          ! checked all combs
         else
            j12(1) = j12(1) + 1
            goto 100
         endif
      endif

      nptsin(1) = 0                         ! current # of constraints involved
      jlowp(1,1) = 1
      do i = 1, jpts(1)-1
         if (pairs(i,jpts(0)) .gt. 0 .and.
     &       pairs(i,jpts(1)) .gt. 0) then
            ptsin(i,1) = 1
            nptsin(1) = nptsin(1) + 1
            lpindx(nptsin(1),1) = i
         else
            ptsin(i,1) = 0
         endif
      enddo
      ptsin(jpts(1),1) = 0
      jlnupts(1) = nptsin(1) + 1 !starting index of pts in lpindx for 1-pt test
      do i = jpts(1)+1, jhgh(1)
         if (pairs(i,jpts(0)) .gt. 0 .and.
     &       pairs(i,jpts(1)) .gt. 0) then
            ptsin(i,1) = 1
            nptsin(1) = nptsin(1) + 1
            lpindx(nptsin(1),1) = i
         else
            ptsin(i,1) = 0
         endif
      enddo
      jhghp(1,1) = nptsin(1)

      if (nptsin(1)-jlnupts(1) .lt. irep(1)-2) then     ! cannot be extended,
                                                        ! need irep(j)+1 points
         if (j12(1) .eq. jcmbs(1)) then
            return                                          ! checked all combs
         else
            j12(1) = j12(1) + 1
            goto 100
         endif
      endif

      do j = 2, ns
         jlowp(j,1) = nptsin(1) + 1
         do i = jlow(j), jhgh(j)
            if (pairs(i,jpts(0)) .gt. 0 .and.
     &          pairs(i,jpts(1)) .gt. 0) then
               ptsin(i,1) = 1
               nptsin(1) = nptsin(1) + 1
               lpindx(nptsin(1),1) = i
            else
               ptsin(i,1) = 0
            endif
         enddo
         jhghp(j,1) = nptsin(1)

         if (jhghp(j,1)-jlowp(j,1) .lt. irep(j)) then      ! cannot be extended
                                                        ! need irep(j)+1 points
            if (j12(1) .eq. jcmbs(1)) then
               return                                       ! checked all combs
            else
               j12(1) = j12(1) + 1
               goto 100
            endif
         endif
      enddo

!     For elimination
      DO i = 1, n
         x(i) = ia(i,jpts(0)) - ia(i,jpts(1))
      ENDDO
      x(n+1) = lft(jpts(1)) - lft(jpts(0))
      DO kout = n, 1, -1
         IF(ABS(x(kout)) .GT. smallnum) GOTO 120
      ENDDO
      print*, 'two pts are same, ith=1, wrong'
      stop
 120  DO i = 1, kout-1
         x(i) = x(i)/x(kout)
      ENDDO
      x(n+1) = x(n+1)/x(kout)
      x(kout) = 1.0d0

!     To form the coefficient matrix A and constant term B of A X <= B.

      do i = jlowp(1,1), jhghp(1,1)
         j = lpindx(i,1)
         b(j) = lft(j) - lft(jpts(0))
         DO i1 = 1, n
            a(i1,j) = ia(i1,jpts(0)) - ia(i1,j)
         ENDDO

         IF(ABS(a(kout,j)) .GT. smallnum) THEN                   ! to eliminate
            DO i1 = 1, kout-1
               a(i1,j) = a(i1,j) - a(kout,j)*x(i1)
            ENDDO
            b(j) = b(j) - a(kout,j)*x(n+1)
         ENDIF

         DO i1 = kout, n-1                                           ! to shift
            a(i1,j) = a(i1+1,j)
         ENDDO

         a(n,j) = 0         ! for added variable alpha0 in the support followed

      enddo

      if (ns .gt. 1) then
         do i = jlowp(2,1), jhghp(ns,1)        ! <\hat a, \hat alpha> >= alpha0
            j = lpindx(i,1)
            b(j) = lft(j)
            DO i1 = 1, n
               a(i1,j) = -ia(i1,j)
            ENDDO

            IF(ABS(a(kout,j)) .GT. smallnum) THEN                ! to eliminate
               DO i1 = 1, kout-1
                  a(i1,j) = a(i1,j) - a(kout,j)*x(i1)
               ENDDO
               b(j) = b(j) - a(kout,j)*x(n+1)
            ENDIF

            DO i1 = kout, n-1                                        ! to shift
               a(i1,j) = a(i1+1,j)
            ENDDO

            a(n,j) = 1                              ! for added variable alpha0
         enddo
      endif

      ith  = 1
      is = ith2is(ith)                   ! do 1-point test in the is-th support
      is_nx = ith2is(ith+1)
      na = ith2na(ith)
      flag = 0

!     To do 1-point test for forming ith+1 point combs
 
      DO WHILE (flag .EQ. 0)
 
*        To calculate the initial x, ib, binv from the info saved

         j_s = jlow_s(ith) + jpter_s(jlow_s(ith)+j12(ith))   
                                                ! starting index of saved info
         do i = 1, na+1
            ib(i) = ib_s(i,j_s)
            if (ib(i) .eq. jpts(ith)) then
               k = i
               goto 130
            endif
         enddo
         print*, 'no index match for reuse, wrong'
         stop
 130     do i = k+1, na+1
            ib(i-1) = ib_s(i,j_s)
         enddo

         do i = 1, na
            if (ptsin(ib(i),ith) .eq. 0) ib(i) = 0    ! the eq. was eliminated
         enddo
         do i = 1, kout-1
            x(i) = x_s(i,j_s)
         enddo
         do i = kout+1, na+1
            x(i-1) = x_s(i,j_s)
         enddo
         i1 = jlow_s(ith)*n + (jpter_s(jlow_s(ith)+j12(ith))-1)*(na+1) 
         do j = 1, k-1
            do i = 1, kout-1
               binv(i,j) = binv_s(i,j+i1)
            enddo
            do i = kout+1, na+1
               binv(i-1,j) = binv_s(i,j+i1)
            enddo
         enddo
         do j = k+1, na+1
            do i = 1, kout-1
               binv(i,j-1) = binv_s(i,j+i1)
            enddo
            do i = kout+1, na+1
               binv(i-1,j-1) = binv_s(i,j+i1)
            enddo
         enddo
               
         if (npn(ith) .gt. 0) then         ! need more points from same support
            jlindx = jlnupts(ith)
            jhindx = jhghp(is,ith)
            nac = na
            p0 = jpts(ith)
         else
            jlindx = jlowp(is+1,ith) ! do the 1st round 1-pt test for next supp
            jhindx = jhghp(is+1,ith)
            nac = na + 1
            p0 = jlow(is_nx) - 1

            x(nac) = 1.0d150
            ell = 0
            do i = jlindx, jhindx
               j = lpindx(i,ith)
               dtmp = b((ith-1)*jh+j)
               do i1 = 1, na
                  dtmp = dtmp - a(i1,(ith-1)*jh+j)*x(i1)
               enddo
               if (x(nac) .gt. dtmp) then
                  ell = j
                  x(nac) = dtmp
               endif
            enddo
            if (ell .eq. 0) then
               print*, 'no ell, wrong. dmxcll.f '
               stop
            endif
            ib(nac) = ell    ! the constraint becomes the last element of base
            do i = 1, na                           ! update the inverse matrix
               binv(i,nac) = 0.0d0
            enddo
            do j = 1, na
               dtmp = 0.0d0
               do i = 1, na
                  dtmp = dtmp + a(i,(ith-1)*jh+ell)*binv(i,j)
               enddo
               binv(nac,j) = -dtmp
            enddo
            binv(nac,nac) = 1.0d0

         endif

         do i = p0+1, jhgh(is_nx)
            ptsin(i,ith) = 0
         enddo

*        To extend the cells by using 1-point test

         p1 = lpindx(jlindx,ith)
         do i = 1, nac
            c(i) = -a(i,(ith-1)*jh+p1)
         enddo

         call dnulp2_a(lda,nac,a(1,(ith-1)*jh+1),
     &                 b((ith-1)*jh+1),c,lpindx(1,ith),
     &                 jhindx,ib,x,binv,v,info)

*        Found a nondegenerate solution for 1-point tests of the points
*        x, ib, binv are available now
*        To record any possilbe points in ib  passed 1-point test

         nnuptsin = 0
         do i = 1, nac
            if (ib(i) .ge. p1) then
               nnuptsin = nnuptsin + 1
               ptsin(ib(i),ith) = 1
            endif
         enddo
         if (nnuptsin .gt. 0) then
            do i = 1, nac
               x_s(i,jlow_s(ith+1)+1) = x(i)
               ib_s(i,jlow_s(ith+1)+1) = ib(i)
            enddo
            do j = 1, nac
               do i = 1, nac
                  binv_s(i,j+n*jlow_s(ith+1)) = binv(i,j)
               enddo
            enddo
         endif

*        To do the 1-point test for other points

         if (info .ge. 0) then
            do j = jlindx+1, jhindx
               p1 = lpindx(j,ith)
               if (ptsin(p1,ith) .eq. 0) then
                  do i = 1, nac
                     c(i) = -a(i,(ith-1)*jh+p1)
                  enddo
                  call dlp1_1pts(lda,nac,a(1,(ith-1)*jh+1),
     &                           b((ith-1)*jh+1),c,lpindx(1,ith),
     &                           jhindx,p1,
     &                           ptsin(1,ith),nnuptsin,
     &                           ib,x,binv,
     &                           x_s(1,jlow_s(ith+1)+1),
     &                           ib_s(1,jlow_s(ith+1)+1),
     &                           binv_s(1,n*jlow_s(ith+1)+1))
               endif
            enddo
         else
            do j = jlindx+1, jhindx
               p1 = lpindx(j,ith)
               if (ptsin(p1,ith) .eq. 0) then
                  do i = 1, nac
                     c(i) = -a(i,(ith-1)*jh+p1)
                  enddo

                  call dlp2_1pts(lda,nac,a(1,(ith-1)*jh+1),
     &                           b((ith-1)*jh+1),c,lpindx(1,ith),
     &                           jhindx,p1,
     &                           ptsin(1,ith),nnuptsin,
     &                           ib,x,binv,v,
     &                           x_s(1,jlow_s(ith+1)+1),
     &                           ib_s(1,jlow_s(ith+1)+1),
     &                           binv_s(1,n*jlow_s(ith+1)+1))
               endif
            enddo
         endif

         if ((npn(ith) .gt. 0 .and. nnuptsin .lt. npn(ith)) .or.
     &     (npn(ith) .eq. 0 .and. nnuptsin .lt. npn(ith+1)+1)) then 
                                               ! cannot be extended to a facet
            if (ith .eq. 1) then
               if (j12(1) .eq. jcmbs(1)) then
                  return
               else
                  j12(1) = j12(1) + 1
                  goto 100
               endif
            else
               flag = 1
               goto 300
            endif
         endif

*        To record the new extendable points

         i = 0
         j = p0
         do while (i .lt. nnuptsin)
            flag = 0
            do while (flag .eq. 0)
               j = j + 1
               if (ptsin(j,ith) .gt. 0) flag = 1
            enddo
            i = i + 1
            lpindx(i,ith+1) = j                      ! used for temporary space

            jpter_s(jlow_s(ith+1)+i) = ptsin(j,ith)
         enddo

         if (ith .eq. mptinc-2) then
            do i = 1, nnuptsin
               do j = 0, mptinc-1
                  ptscmb(ncmb*mptinc+j+1) = jpts(j)
               enddo
               ptscmb(ncmb*mptinc+mptinc) = lpindx(i,ith+1)
               ncmb = ncmb + 1
            enddo
            flag = 1                                            ! To reduce ith
c           print*, 'ncmb : ', ncmb
            goto 300
         else
            j = jlowc(ith+1)
            do i = 1, nnuptsin
               j = j + 1
               ptscmb1(j) = lpindx(i,ith+1)
            enddo                                   ! move this into dlp_1pt.f?
            jcmbs(ith+1) = nnuptsin
            jlowc(ith+2) = jlowc(ith+1) + nnuptsin
            jlow_s(ith+2) = jlow_s(ith+1) + nnuptsin
            flag = 0
         endif

 300     IF(flag .EQ. 1) THEN               ! The comb is not in the lower hull
            IF(j12(ith) .EQ. jcmbs(ith)) THEN
                                    ! No pairs left in ith support. Back search
               ith = ith - 1
               DO WHILE(flag .EQ. 1 .AND. ith .GT. 0)
                  IF(j12(ith) .LT. jcmbs(ith)) THEN
                     flag = 0
                  ELSE
                     ith  = ith - 1
                  ENDIF
               ENDDO
               IF(flag .EQ. 1) THEN
                  RETURN                              ! have verified all combs
               ENDIF
               j12(ith) = j12(ith) + 1  
               IF(ith .EQ. 1) THEN
                  GOTO 100                         ! Start from the 1st support
               ENDIF
            ELSE
               j12(ith) = j12(ith) + 1 
            ENDIF
            flag = 0
         ELSE                                   ! The comb is in the lower hull
            ith = ith + 1
            j12(ith) = 1
         ENDIF

*        To set up the new a, b, lpindx(1,ith), nptsin(ith),

         ith_ = ith - 1
         na = ith2na(ith)
         is = ith2is(ith)
         is_ = ith2is(ith_)
         is_nx = ith2is(ith+1)

         jpts(ith) =  ptscmb1(jlowc(ith)+j12(ith))
         if (jhgh(is)-jpts(ith) .lt. npn(ith)) then      
                                                ! cannot be extended to a facet
            flag = 1
            goto 300
         endif

         nptsin(ith) = 0                    ! current # of constraints involved
         do j = 1, is-1
            jlowp(j,ith) = nptsin(ith) + 1
            do i = jlow(j), jhgh(j)
               if (pairs(i,jpts(ith)) .gt. 0 .and.
     &             ptsin(i,ith_) .gt. 0) then
                  ptsin(i,ith) = 1 
                  nptsin(ith) = nptsin(ith) + 1
                  lpindx(nptsin(ith),ith) = i
               else
                  ptsin(i,ith) = 0
               endif
            enddo
            jhghp(j,ith) = nptsin(ith)
         enddo

         jlowp(is,ith) = nptsin(ith) + 1
         do i = jlow(is), jpts(ith)-1
            if (pairs(i,jpts(ith)) .gt. 0 .and.
     &          ptsin(i,ith_) .gt. 0) then
               ptsin(i,ith) = 1 
               nptsin(ith) = nptsin(ith) + 1
               lpindx(nptsin(ith),ith) = i
            else
               ptsin(i,ith) = 0
            endif
         enddo
         ptsin(jpts(ith),ith) = 0
         jlnupts(ith) = nptsin(ith) + 1
         do i = jpts(ith)+1, jhgh(is)
            if (pairs(i,jpts(ith)) .gt. 0 .and.
     &          ptsin(i,ith_) .gt. 0) then
               ptsin(i,ith) = 1 
               nptsin(ith) = nptsin(ith) + 1
               lpindx(nptsin(ith),ith) = i
            else
               ptsin(i,ith) = 0
            endif
         enddo
         jhghp(is,ith) = nptsin(ith)

         if (nptsin(ith)-jlnupts(ith) .lt. npn(ith)-1) then
            flag = 1
            goto 300
         endif

         do j = is+1, ns
            jlowp(j,ith) = nptsin(ith) + 1
            do i = jlow(j), jhgh(j)
               if (pairs(i,jpts(ith)) .gt. 0 .and.
     &             ptsin(i,ith_) .gt. 0) then
                  ptsin(i,ith) = 1 
                  nptsin(ith) = nptsin(ith) + 1
                  lpindx(nptsin(ith),ith) = i
               else
                  ptsin(i,ith) = 0
               endif
            enddo
            jhghp(j,ith) = nptsin(ith)

            if (jhghp(j,ith)-jlowp(j,ith) .lt. irep(j)) then   
                                    ! cannot be extended, need irep(j)+1 points
               flag = 1
               goto 300
            endif
         enddo

!        elimination

         i1 = jh*(ith-2)
         i2 = jh*(ith-1)

         DO i = 1, na+1
            x(i) = a(i,i1+jpts(ith))
         ENDDO
         x(n+1) = b(i1+jpts(ith))

         DO kout = na+1, 1, -1
            IF(ABS(x(kout)) .GT. smallnum) GOTO 320
         ENDDO
         print*, 'two points are same, ith>1, wrong'
         stop
 320     DO i = 1, kout-1
            x(i) = x(i)/x(kout)
         ENDDO
         x(n+1) = x(n+1)/x(kout)
         x(kout) = 1.0d0


!        To eliminate the kout-th varaible and shift rows up

         if (is .eq. is_) then                         ! still in same support
            do i = jlowp(1,ith), jhghp(ns,ith)
               j = lpindx(i,ith)
               IF(ABS(a(kout,j+i1)) .GT. smallnum) THEN         ! to eliminate
                  b(j+i2) = b(j+i1) - a(kout,j+i1)*x(n+1)
                  DO j1 = 1, kout-1
                     a(j1,j+i2) = a(j1,j+i1) - a(kout,j+i1)*x(j1)
                  ENDDO
               ELSE
                  b(j+i2) = b(j+i1)
                  do j1 = 1, kout-1
                     a(j1,j+i2) = a(j1,j+i1)
                  enddo
               ENDIF
 
               DO j1 = kout, na                                     ! to shift
                  a(j1,j+i2) = a(j1+1,j+i1)
               ENDDO
            enddo

            do i = jlowp(1,ith), jhghp(is,ith)
               j = lpindx(i,ith)
               a(na+1,j+i2) = 0               ! since no added variable alpha0
            enddo
            if (is .lt. ns) then
               do i = jlowp(is+1,ith), jhghp(ns,ith)
                  j = lpindx(i,ith)
                  a(na+1,j+i2) = 1                 ! for added variable alpha0
               enddo
            endif

         else                         ! fixing the first point in a new support

            do i = jlowp(1,ith), jhghp(ns,ith)   ! copy the previous a, b saved
               j = lpindx(i,ith)
               do j1 = 1, na+1
                  a(j1,j+i2) = a(j1,j+i1)
               enddo
               if (j .le. jhgh(is)) then
                  a(na+1,j+i2) = 0
               else
                  a(na+1,j+i2) = 1
               endif
               b(j+i2) = b(j+i1)
            enddo

            do i = jlowp(is,ith), jhghp(is,ith)        ! eliminate the alpha0 
               j = lpindx(i,ith)
               do j1 = 1, na
                  a(j1,j+i2) = a(j1,j+i2) - x(j1)
               enddo
               b(j+i2) = b(j+i2) - x(n+1)
            enddo

         endif

      ENDDO
      END
