      SUBROUTINE dpairs(lda,ldb,n,ns,ia,jlow,jhgh,lft,
     &                  pairs,
     &                  lpindx,
     &                  ib,a,b,c,binv,x,v,
     &                  x_s,ib_s,binv_s)

!     This is to fill out the relation table as described in the paper
!     "Mixed Volume Computation For Semi-mixed Systems" by Tangan Gao
!     and T.Y. Li

      IMPLICIT NONE

!  .. Input and Outpu Variables

!  .. Scalar variables ..
 
      integer          lda, ldb, n, ns
 
!  .. Array variables ..
 
      integer          ia(lda,*), jlow(*), jhgh(*)
      double precision lft(*) 

      integer          pairs(ldb,*)
 
! INPUT:
!     n   :  # of variables of the system
!     ns  :  # of different supports of the system
!     ia  : ia(1:n,1:jhgh(ns)) stores the support of the semi-mixed system
!     jlow(1:ns), jhgh(1:ns), lft : as usual

! OUTPUT:
!     pairs : pairs(i,j) = 1 means (i,j) is a lower edge. 
! x_s,ib_s,binv_s: saved info for starting feasible points of LP problems
 
!  .. Local scalar variables ..
 
      INTEGER          i, j, j1, nptsin, jlnupts, p1, p2, is, is_nx, 
     &                 info, ell, flag
      double precision dtmp
 
!  .. Local array variables ..
 
      INTEGER          lpindx(*), ib(*)
      DOUBLE PRECISION a(lda,*), b(*), binv(lda,*), x(*), c(*), v(*)

!     The next four variables are used to save info for starting feasible
!     points of the LP problems:

      integer          n_s, ib_s(lda,*)
      double precision x_s(lda,*), binv_s(lda,*)

!  .. Executable code ..

      do j = 1, jhgh(ns)
         do i = 1, jhgh(ns)
            pairs(i,j) = 0
         enddo
      enddo
      do j = 1, jhgh(ns)
         do i = 1, n
            a(i,j) = -ia(i,j)
         enddo
         a(n+1,j) = 1
         b(j) = lft(j)
      enddo

      n_s = 0                                       ! # of groups of info saved 

      Do is = 1, ns
         do p1 = jlow(is), jhgh(is)
            nptsin = 0                              ! # of constraints involved
            do i = jlow(is), p1-1
               if (pairs(i,p1) .gt. 0) then
                  nptsin = nptsin + 1
                  lpindx(nptsin) = i
               endif
            enddo
            jlnupts = nptsin + 1          ! starting index of pts for 1-pt test
            do i = p1+1, jhgh(is)
               nptsin = nptsin + 1
               lpindx(nptsin) = i
            enddo

!           To form the coefficient matrix A and constant term B of A X <= B

            do j1 = 1, nptsin
               j = lpindx(j1)
               b(j) = lft(j) - lft(p1)
               DO i = 1, n
                  a(i,j) = ia(i,p1) - ia(i,j)
               ENDDO
               a(n+1,j) = 1
            enddo

!           To extend the point by using 1-point test

!           To find a nondegenerate solution for 1-point tests of the points

            call dnulp2_e(lda,n,jhgh(ns),a,b,lpindx,nptsin,ib,x,
     &                    binv,c,info)

            if (p1 .eq. jhgh(is)) goto 100

!           To record any possilbe points in ib  passed 1-point test

            if (is .eq. 1) then
               flag = 0
               do i = 1, n
                  if (ib(i) .gt. p1 .and. 
     &                pairs(ib(i),p1) .eq. 0) then
                     flag = 1
                     pairs(ib(i),p1) = n_s + 1
                     pairs(p1,ib(i)) = n_s + 1
                  endif
               enddo
               if (flag .eq. 1) then
                  n_s = n_s + 1
                  do i = 1, n
                     x_s(i,n_s) = x(i)
                     ib_s(i,n_s) = ib(i)
                  enddo
                  do j = 1, n
                     do i = 1, n
                        binv_s(i,j+n*(n_s-1)) = binv(i,j)
                     enddo
                  enddo
               endif
            else
               do i = 1, n
                  if (ib(i) .gt. p1) then
                     pairs(ib(i),p1) = 1
                     pairs(p1,ib(i)) = 1
                     do j = i+1, n
                        if (ib(j) .gt. p1) then 
                           pairs(ib(i),ib(j)) = 1
                           pairs(ib(j),ib(i)) = 1
                        endif
                     enddo
                  endif
               enddo
            endif

!           x, ib, binv are available now
!           To do the 1-point test for other points

            if (info .ge. 0) then
               do j = jlnupts, nptsin
                  p2 = lpindx(j)
                  if (pairs(p2,p1) .eq. 0) then
                     do i = 1, n
                        c(i) = -a(i,p2)
                     enddo
                     if (is .eq. 1) then
                        call dlp1_1pt_s(lda,ldb,n,a,b,c,lpindx,
     &                                  nptsin,p1,p2,
     &                                  pairs,
     &                                  ib,x,binv,
     &                                  n_s,x_s,ib_s,binv_s)
                     else
                        call dlp1_1pt_i(lda,ldb,n,a,b,c,lpindx,
     &                                  nptsin,p1,p2,
     &                                  pairs,
     &                                  ib,x,binv)
                     endif
                  endif
               enddo
            else
               do j = jlnupts, nptsin
                  p2 = lpindx(j)
                  if (pairs(p2,p1) .eq. 0) then
                     do i = 1, n
                        c(i) = -a(i,p2)
                     enddo
                     if (is .eq. 1) then
                        call dlp2_1pt_s(lda,ldb,n,a,b,c,lpindx,
     &                                  nptsin,p1,p2,
     &                                  pairs,
     &                                  ib,x,binv,v,
     &                                  n_s,x_s,ib_s,binv_s)
                     else
                        call dlp2_1pt_i(lda,ldb,n,a,b,c,lpindx,
     &                                  nptsin,p1,p2,
     &                                  pairs,
     &                                  ib,x,binv,v)
                     endif
                  endif
               enddo
            endif

 100        do j = 1, n                   ! save the starting point for next LP
               do i = 1, n              ! for dpairs.f only, not for extension!
                  binv_s(i,j+n_s*n) = binv(i,j)
               enddo
               ib_s(j,n_s+1) = ib(j)
               x_s(j,n_s+1) = x(j)
            enddo
               
!           To check each point in S_is and each point in S_i (i>is)

            nptsin = 0                              ! # of constraints involved
            do i = jlow(is), jhgh(is)
               if (pairs(i,p1) .gt. 0) then
                  nptsin = nptsin + 1
                  lpindx(nptsin) = i
               endif
            enddo

!           To form  A and B of A X <= B WRT S_is

            do j1 = 1, nptsin
               j = lpindx(j1)
               a(n+1,j) = 0                  ! no alpha_0 for these constraints
            enddo                            ! others stay same

            jlnupts = nptsin + 1          ! starting index of pts for 1-pt test

            do is_nx = is+1, ns

               nptsin = jlnupts - 1
               do i = jlow(is_nx), jhgh(is_nx)
                  nptsin = nptsin + 1
                  lpindx(nptsin) = i
               enddo

!              The part of Ax<=B for this support were formed at the beginning

!              To do the 1-point test for the points in S_(is_nx)

!              To form the starting point from saved starting ib, x, binv

               ell = 0
               x(n+1) = 1.0d150
               do j1 = jlnupts, nptsin
                  j = lpindx(j1)
                  dtmp = b(j)
                  do i = 1, n
                     dtmp = dtmp - a(i,j)*x_s(i,n_s+1)
                  enddo
                  if (dtmp .lt. x(n+1)) then
                     x(n+1) = dtmp
                     ell = j
                  endif
               enddo
               do i = 1, n
                  ib(i) = ib_s(i,n_s+1)
                  x(i) = x_s(i,n_s+1)
               enddo
               ib(n+1) = ell
               do j = 1, n
                  do i = 1, n
                     binv(i,j) = binv_s(i,j+n*n_s)
                  enddo
                  binv(n+1,j) = 0.0d0
               enddo
               do i = 1, n
                  binv(i,n+1) = 0.0d0
               enddo
               do j = 1, n                ! the last row = a(*,ell)*binv(*,1:n)
                  binv(n+1,j) = 0.0d0
                  do i = 1, n
                     binv(n+1,j) = binv(n+1,j) - binv(i,j)*a(i,ell)
                  enddo
               enddo
               binv(n+1,n+1) = 1.0d0

!              To find a nondegenerate solution for 1-point tests of the points

               p2 = lpindx(jlnupts)
               do i = 1, n+1
                  c(i) = -a(i,p2)
               enddo
               call dnulp2_a(lda,n+1,a,b,c,lpindx,nptsin,ib,x,
     &                       binv,v,info)

!              To record any possilbe points in ib passed 1-point test

               do i = 1, n+1
                  if (ib(i) .ge. p2) then 
                        pairs(ib(i),p1) = 1
                        pairs(p1,ib(i)) = 1
                  endif
                  do j = i+1, n+1
                     if (ib(j) .ge. jhgh(is_nx)) then
                           pairs(ib(i),ib(j)) = 1
                           pairs(ib(j),ib(i)) = 1
                     endif
                  enddo
               enddo

!              To do the 1-point test for other points

               if (info .ge. 0) then
                  do j = jlnupts+1, nptsin
                     p2 = lpindx(j)
                     if (pairs(p2,p1) .eq. 0) then
                        do i = 1, n+1
                           c(i) = -a(i,p2)
                        enddo
                        call dlp1_1pt_i(lda,ldb,n+1,a,b,c,lpindx,
     &                                  nptsin,p1,p2,
     &                                  pairs,
     &                                  ib,x,binv)
                     endif
                  enddo
               else
                  do j = jlnupts+1, nptsin
                     p2 = lpindx(j)
                     if (pairs(p2,p1) .eq. 0) then
                        do i = 1, n+1
                           c(i) = -a(i,p2)
                        enddo
                        call dlp2_1pt_i(lda,ldb,n+1,a,b,c,lpindx,
     &                                  nptsin,p1,p2,
     &                                  pairs,
     &                                  ib,x,binv,v)
                     endif
                  enddo
               endif
            enddo
         enddo
      enddo

      return
      end
