      SUBROUTINE ddet(lda,n,a,
     &                det,
     &                v)
 
      IMPLICIT none

!  .. Extra Variable ..
 
      integer lda
 
!  .. scalar varibales ..
 
      INTEGER          n
      DOUBLE PRECISION det
 
!  .. vector variables
 
      DOUBLE PRECISION a(lda,*), v(*)
 
!       this subroutine is to use Gaussian elimination method to get 
!	the determinant det of the n by n  matrix a(1:n,1:n) and solve 
!       a(1:n,1:n) v = a(:,n+1)
 
!  .. Local variables ..
 
      INTEGER          i, j, k, itmp1
      DOUBLE PRECISION tmp2, eps
 
!  .. Executable code ..
 
      eps=1.0d-14
 
      do i=1,n-1
!        to find the first non-zero elemnet in i-th column for Gauss
         itmp1=i-1
         do j=i,n
            if(abs(a(j,i)) .ge. eps) then
               itmp1=j
!              itmp1-th row has the first nonzero element
               goto 10
            endif
         enddo

         if (itmp1 .eq. i-1 ) then
            print*, 'In det, the matrix a is not of full rank'
            return
         endif

!        exchange the two rows. make the (i,i) element nonzero
 10      if (itmp1 .gt. i) then
            do j=i,n+1
               tmp2=a(itmp1,j)
               a(itmp1,j)=a(i,j)
               a(i,j)=tmp2
            enddo
         endif
!
!        make all other elements in the column zero
         do j=i+1, n
            if(abs(a(j,i)) .ge. eps) then
               do k=i+1, n+1
                  a(j,k)=a(j,k)-a(i,k)*a(j,i)/a(i,i)
               enddo
!              a(j,i)=0.0d0
            endif
         enddo
      enddo 
 
      if (abs(a(n,n)) .le. eps) then
         print*, 'In det, the matrix a is not of full rank'
         return
      endif
 
!     ready to get the det

      det=1.0d0
      do i=1,n
         det=det*a(i,i)
      enddo
      if (abs(det) .lt. eps) then
         print*,'In det, det=0. It is impossible'
      endif
      if (det .lt. 0.0) then
         det = -det
      endif
!
      k = 0
      if(k .eq. 0) then
         v(n+1) = 1.0d0
         do i = n, 1, -1
            v(i) = -a(i,n+1)
            do j = i+1, n
               v(i) = v(i) - a(i,j)*v(j)
            enddo
            v(i) = v(i)/a(i,i)
         enddo 
      endif
!
      return
      end
