      SUBROUTINE mxvl(lda,n,ns,ia,irep,ilow,ihgh,lft,ptscomb,nocomb,
     &                mvl,
     &                work)
 
      IMPLICIT none

!  .. Extra Variables ..

      integer lda
 
!  .. scalar variables
 
      INTEGER n, ns, nocomb, mvl
 
!  .. vector variables
 
      INTEGER ia(lda,*), irep(*), ilow(*), ihgh(*), ptscomb(*)
      DOUBLE PRECISION lft(*)
 
!  .. Local array variables ..
 
      DOUBLE PRECISION  work(lda,*), v(100)
 
!  .. Purpose ..
 
!       This subroutine is to calculate the mixed volume of a poly
!       system after geting all the ptscomb.
!
!  .. Input and Output ..
!
!       ia       :  each column is a point in some support. The first 
!                   coordinate is the value lifted.
!       ns       :  number of different supports.
!       irep     :  the i-th support repeats irep(i) times.
!       lft      :  lft(i) is the lifting value for the i-th point in ia
!       nocomb   :  total number of lower faces of type (1,...,1)
!       ptscomb  :  contains the positions in a of the points for
!                   lower facets of type (1,...,1). ptscomb(2*n*(i-1)+j),
!                   j=1,2,...,2n give the i-th lower face.
!       mvl      :  mixed volume.
 
!  .. local variables ..
 
      INTEGER          i, i1, i2, icnt, j, k, nvtx, 
     &                 itmp1, itmp2, neqs
      DOUBLE PRECISION det, dtmp
 
!  .. Executable code
 
      mvl = 0
      nvtx = ns
      DO i = 1, ns
      nvtx = nvtx + irep(i)                         ! # of points in each facet
      ENDDO
      DO i = 1, nocomb
         icnt = 0
         neqs = 0
         DO i1 = 1, ns
            icnt = icnt + 1
            itmp1 = ptscomb(nvtx*(i-1)+icnt)
            DO j = 1, irep(i1)
               icnt = icnt + 1
               itmp2 = ptscomb(nvtx*(i-1)+icnt)
               neqs = neqs + 1
               DO k = 1, n
                  work(neqs,k) = ia(k,itmp2) - ia(k,itmp1)
               ENDDO
               work(neqs,n+1) = lft(itmp2) - lft(itmp1)
            ENDDO
         ENDDO
!
         CALL ddet(lda,n,work,det,v)
!
         k = 0                        ! In case we need the facets and normals
         IF(k .EQ. 1) THEN
            write(*,*) 
     &         'The i-th comb, mixed volume, points, normal vector'
            write(*,*) 'i = ', i
            write(*,*) 'volume of the cell ', nint(det)
            DO j = 1, nvtx
               write(*,*) ptscomb(nvtx*(i-1)+j)
            ENDDO
            DO j = 1, n+1
               write(*,*) v(j)
            ENDDO
         ENDIF
         IF(k .EQ. 0) THEN           ! For testing whether the cells are right
            DO i1 = 1, ns
               itmp1 = 0
               DO i2 = 1, i1-1
                  itmp1 = itmp1 + irep(i2)
               ENDDO
               itmp1 = ptscomb(nvtx*(i-1)+1+itmp1+i1-1)
               DO j = ilow(i1), ihgh(i1)
                  dtmp = lft(itmp1) - lft(j)
                  DO i2 = 1, n
                     dtmp = dtmp + v(i2)*(ia(i2,itmp1)-ia(i2,j))
                  ENDDO
                  IF(dtmp .GT. 1.0d-8) THEN
                     print*, 'imxvl'
                     print*, i, i1, j, det, dtmp
                  ENDIF
               ENDDO
            ENDDO
         ENDIF
         mvl = mvl + nint(det)
      ENDDO
!
      END
