      SUBROUTINE nlhom_un(lda,n,ia,ns,irep,ap,lft,ilow,ihgh,ptscmb,ncmb,
     &                    nrmlv,roots0,roots1,nrts,ntimes,rtlow,rthgh,
     &                    tpwr,p0,p1,p2,v1,v2,dhz,h,p3,cf)

      IMPLICIT none

!  .. Scalar Arguments ..

      INTEGER lda, n, ns, ncmb, nrts

!  .. Array Arguments ..

      INTEGER ia(lda,*), irep(*), ilow(*), ihgh(*), ptscmb(*),
     &        rtlow(*), rthgh(*), ntimes(*)
      DOUBLE PRECISION lft(*), nrmlv(lda,*)
      COMPLEX*16 ap(*), roots0(lda,*), roots1(lda,*)

!  .. Local array variables ..

      DOUBLE PRECISION tpwr(*)

      COMPLEX*16 p0(*), p1(*), p2(*), v1(*), v2(*),
     &           dhz(lda,*), h(*), p3(*), cf(*)

!  .. PURPOSE ..

!  	This is to get all the roots of a generic polynomial
!       system from the initial polynomial systems defineed by
!       the terms corresponding to the lower facets.

!  .. Input/output ..

! ia      -- integer array of dimension n by tnop+1
!            each column of ia(n,2:tnop+1) is a point in the
!            support of the polynomial sytem
! ap      -- complex*16 array of dimension tnop+1 (input)
!            each component of ap(2:tnop+1) is the coefficient of 
!            the corresponding term in the generic polynomial system
! lft     -- double precision array of dimension tnop+1
!            each component of lft(2:tnop+1) is the lifting value for
!            the corresponding point
! ilow    -- integer array of dimension n
!            the terms of the i-th polynomial begins fron the ilow(i)-th
!            column of ia
! ihgh    -- integer array of dimension n
!            the terms of the i-th polynomial ends at the ihgh(i)-th
!            column of ia
! ptscmb  -- integer array of dimension ? (input)
!            the indecies of the points for the lower facets
! ncmb    -- integer (input)
!            the total number of lower facets
! nrmlv   -- double precision array of dimension n by ? (input)
!            the i-th column is the normal vector of the i-th lower
!            facet(the component of the lifting direction is set to be 
!            one and is not be stored)
! roots0  -- complex*16 array of dimension n by ? (input)
!            the initial roots for the initial systems.
! roots1  -- complex*16 array of dimension n by ? (output)
!            the roots of the target system
! rtlow   -- integer array of diemnsion  ?
!            the roots of the i-th initial system (corresponding to
!            the i-th lower facet) begins from rtlow(i)-th column
!            of roots
! rthgh   -- integer array of diemnsion  ?
!            the roots of the i-th initial system (corresponding to
!            the i-th lower facet) ends at the rthgh(i)-th column
!            of roots
! ntimes  -- ntimes(i) is the # of times the i-th curves being traced.
!
! tpwr    -- real array of dimension tnop+1
!            the powers of $t$ after transformation.

!  .. Local Variables ..

      INTEGER i, i1, i2, is, j, nvtx, isum, yn
      DOUBLE PRECISION dtmp

!  .. Executable Code ..

      yn = ntimes(1)
      nvtx = ns
      DO i = 1, ns
         nvtx = nvtx + irep(i)
      ENDDO
      DO i = rtlow(1), rthgh(ncmb)
         ntimes(i) = 1
      ENDDO
      nrts = 0                               ! to count the number of roots got

      DO i = 1, ncmb

!        For each lower facet, to get the powers of t for all terms 
!        after the transformation as described in Huber's paper

         DO i1 = ilow(1), ihgh(ns)
            tpwr(i1) = lft(i1)
            DO i2 = 1, n
               tpwr(i1) = tpwr(i1) + ia(i2,i1)*nrmlv(i2,i)
            ENDDO
         ENDDO
         isum = 0
         DO i1 = 1, ns
            IF (i1 .GT. 1) isum = isum + irep(i1-1)
            dtmp = tpwr(ptscmb(nvtx*(i-1)+isum+i1))
            DO i2 = ilow(i1), ihgh(i1)
               tpwr(i2) = tpwr(i2) - dtmp
            ENDDO
         ENDDO
         DO i1 = 1, nvtx
            tpwr(ptscmb(nvtx*(i-1)+i1)) = 0.0d0
         ENDDO
         is = ns
         DO i1 = 1, ns
            DO j = 2, irep(i1)
               is = is + 1
               DO i2 = 0, ihgh(i1)-ilow(i1)
                  tpwr(ilow(is)+i2) = tpwr(ilow(i1)+i2)
               ENDDO
            ENDDO
         ENDDO

         IF (yn .EQ. 0) THEN   !lftng was not called
            dtmp= 1.0d150    !! not necessary if lftng was called
            DO i1 = 1, n
               DO j = ilow(i1), ihgh(i1)
                  IF(tpwr(j) .LT. dtmp .AND. tpwr(j) .GT. 1.0d-9) 
     &               dtmp = tpwr(j)
               ENDDO
            ENDDO
            DO j = ilow(1), ihgh(n)
               tpwr(j) = tpwr(j)/dtmp 
            ENDDO
         ENDIF

         CALL znpath1(lda,n,ilow,ihgh,ap,tpwr,roots0,i,rtlow,rthgh,
     &                roots1,nrts,ntimes,
     &                p0,p1,p2,v1,v2,dhz,h,p3,cf)

      ENDDO

      RETURN
      END
