      SUBROUTINE z1step1(lda,n,ilow,ihgh,ap,tpwr,p_old,v_old,
     &                   p_new,v_new,t_new,stp_new,stp_c,info,
     &                   dhz,h,p0,cf)

      IMPLICIT none

      include '../global.h'

!  .. Scalar Arguments ..

      INTEGER          lda, n, info
      DOUBLE PRECISION t_new, stp_new, stp_c

!  .. Array Arguments ..

      INTEGER          ilow(*), ihgh(*)
      DOUBLE PRECISION tpwr(*)
      COMPLEX*16       ap(*), p_old(*), p_new(*), v_old(*), v_new(*)

!  .. Local array variables ..

      COMPLEX*16 dhz(lda,*), h(*), p0(*), cf(*)

!  .. PURPOSE ..

!       this is one step of following a curve from two previous
!       points known until to find next point on the curve.
!       after find the point, it will renew p_*,v_*,stp_*.

!  .. Input/Output ..

! ap            The coefficients of the system.
! p_old, p_new  The last two consecutive points known on the curve.
! v_old, v_new  The tangent vectors at the point p_old and p_new.
! stp_new       The step size to get p_new from p_old.
! stp_c         The current step size to predict the next point.
! info= 2       Newton's iteration divergens.
!     = 1       Coefficient matrix dhz is singular.
!     = 0       Newton's iteration converges.
!     = -1      Got a root at t=1
!
! dhz           The Jacobian matrix for Newton's method
! h             The function values for Newton's method
! p0            The points generated by Newton's method

!  .. Local Variables ..

      INTEGER          i,iter
      DOUBLE PRECISION nrm_p0, nrm_p1, nrm_dif, dtmp

      DOUBLE PRECISION dznrm2

!  .. Executable code ..

      IF(t_new+stp_c .gt. 1.0d0-gstp_min) THEN

!        iteration on level t=1

         stp_c = 1.0d0 - t_new

         CALL zhermite3(n,p_old,v_old,stp_new,p_new,v_new,stp_c,p0)

         CALL znwtnat11(lda,n,ap,p0,info,dhz,h)
         IF(info .eq. 0) THEN
            DO i = 1, n
               p_new(i) = p0(i)                        ! p_new is the solution
            ENDDO
            info = -1
            RETURN
         ELSE
            stp_c = stp_c/2.0d0
            GOTO 10
         ENDIF
      ENDIF

!     To predict next point p0 near the curve

 10   CALL zhermite3(n,p_old,v_old,stp_new,p_new,v_new,stp_c,p0)

      DO i = ilow(1), ihgh(n)
         cf(i) = ap(i)*(t_new+stp_c)**tpwr(i)
      ENDDO
      CALL zgetp1(cf,p0,h)
      nrm_p0 = dznrm2(n,h)

      iter = 0
      DO WHILE(iter .le. gmax_nwtn) 

!        One step of Newton's iteration

         CALL zgetdp1(lda,cf,p0,dhz)
         CALL cqrdc_slv(lda,n,dhz,h,info)
         IF(info .eq. 1) RETURN                               ! dhz is singular
         DO i = 1, n
            p0(i) = p0(i) - h(i)
         ENDDO
         nrm_dif = dznrm2(n,h)

         CALL zgetp1(cf,p0,h)
         nrm_p1 = dznrm2(n,h)

         dtmp = 10.0d0
         DO i = 1, n
            dtmp = max(dtmp,ABS(DBLE(p0(i)))+ABS(DIMAG(p0(i))))
         ENDDO
         dtmp = MIN(dtmp,1.0d5)  !! To keep dtmp not too big
         IF(nrm_p1/dtmp .lt. geps_trace*10  
     &      .OR. nrm_dif .lt. geps_at1*100 ) THEN

!           To store the information for next step

            DO i = 1, n
               p_old(i) = p_new(i)
            ENDDO
            DO i = 1, n
               p_new(i) = p0(i)
            ENDDO
            DO i = 1, n
               v_old(i) = v_new(i)
            ENDDO
            t_new = t_new + stp_c

!           To get the new tangent vector at p_new

            CALL zgetdp1(lda,cf,p_new,dhz)
            DO i = ilow(1), ihgh(n)
               cf(i) = tpwr(i)*cf(i)/t_new
            ENDDO
            CALL zgetp1(cf,p_new,v_new)
            DO i = 1, n
               v_new(i) = -v_new(i)
            ENDDO
            CALL cqrdc_slv(lda,n,dhz,v_new,info)
            IF(info .eq. 1) RETURN                            ! dhz is singular

            info = 0
            RETURN
         ENDIF

         IF ( (iter .eq. 0 .or. iter .eq. 1) .AND. 
     &         nrm_p1/nrm_p0 .ge. grsdl_fctr) GOTO 20 

         nrm_p0  = nrm_p1
         iter    = iter + 1
      ENDDO
 
 20   stp_c = stp_c/2.0d0
      IF(stp_c .lt. gstp_min) THEN
         info = 2
         RETURN
      ELSE
         GOTO 10
      ENDIF

      END
