      SUBROUTINE z1ststp1(lda,n,ilow,ihgh,ap,tpwr,p_old,stp_c,p_new,
     &                    v_new,info,
     &                    dhz,h,cf)

      IMPLICIT none

      include '../global.h'

!  .. Scalar Arguments ..

      INTEGER lda, n, info
      DOUBLE PRECISION stp_c

!  .. Array Arguments ..

      INTEGER ilow(*), ihgh(*)
      DOUBLE PRECISION tpwr(*)
      COMPLEX*16 ap(*), p_old(*), p_new(*), v_new(*)

!  .. Local array variables ..

      COMPLEX*16 dhz(lda,*), h(*), cf(*)

!  .. PURPOSE ..

!       This is first step of following a curve from initial
!       point known until to find next point on the curve.
!       after find the point, the two points known will be stored
!       in  p_old and p_new. The corresponding tangent vector will
!       be v_new at p_new.

!  .. Input/Output ..

! p_old   The initial point on on the curve.
! p_new   The iteration points of Newton's method.
!         The new point on the curve found on exit.
! v_new   The tangent vector at p_new.
! stp_c   The current step size to predict the next point
! info= 2 Newton's iteration divergens. 
!     = 1 Coefficient matrix dhz is singular.
!     = 0 Newton's iteration converges.
!
! dhz     The Jacobian matrix at p_new.
! h       The function values at p_new. 
 
!  .. Local Variables ..
 
      INTEGER  i, iter
      DOUBLE PRECISION nrm_p0, nrm_p1, nrm_dif

      DOUBLE PRECISION dznrm2
 
!  .. Executable code ..
 
!     Linear predict at t=0
 
 10   DO i = 1, n
         p_new(i) = p_old(i)
      ENDDO

      DO i = ilow(1), ihgh(n)
         cf(i) = ap(i)*stp_c**tpwr(i)
      ENDDO
      CALL zgetp1(cf,p_new,h)
      nrm_p0 = dznrm2(n,h)

      iter = 0
      DO WHILE(iter .le. gmax_nwtn) 
 
!        One step Newton's iteration
 
         CALL zgetdp1(lda,cf,p_new,dhz)
         CALL cqrdc_slv(lda,n,dhz,h,info)
         IF(info .eq. 1) RETURN                              ! dhz is singular
         DO i = 1, n
            p_new(i) = p_new(i) - h(i)
         ENDDO
         nrm_dif = dznrm2(n,h)

         CALL zgetp1(cf,p_new,h)
         nrm_p1 = dznrm2(n,h)

         IF(nrm_p1 .lt. geps_trace .OR. (nrm_p1 .lt. geps_trace*100 
     &      .AND. nrm_dif .lt. geps_at1) ) THEN
 
!           To get the new tangent vector at p_new
 
            CALL zgetdp1(lda,cf,p_new,dhz)
            DO i = ilow(1), ihgh(n)
               cf(i) = tpwr(i)*cf(i)/stp_c
            ENDDO
            CALL zgetp1(cf,p_new,v_new)
            DO i = 1, n
               v_new(i) = -v_new(i)
            ENDDO
            CALL cqrdc_slv(lda,n,dhz,v_new,info)
            IF(info .eq. 1) RETURN

            info = 0
            RETURN
         ENDIF

         IF(nrm_p1/nrm_p0 .ge. grsdl_fctr) GOTO 20  !!residual ratio is too big

         nrm_p0  = nrm_p1
         iter    = iter + 1
      ENDDO
 
 20   stp_c = stp_c/2.0d0
      IF(stp_c .lt. gstp_min) THEN
         info = 2                                  !! step size is too small
         RETURN
      ELSE
         GOTO 10
      ENDIF

      END
