      SUBROUTINE znwtnat11(lda,n,ap,p0,info,
     &                     dhz,h)

      IMPLICIT none

      include '../global.h'

!  .. Scalar Arguments ..

      INTEGER lda, info, n

!  .. Array Arguments ..

      COMPLEX*16 ap(*), p0(*)
      COMPLEX*16 h(*), dhz(lda,*)

!  .. PURPOSE ..

!	The Newton iteration on the level t=1

!  .. Input and output ..

! ap      The coefficients of the system
! p0      The initial point for Newton's iteration
! info= 2 Newton's iteration divergens.
!     = 1 Coefficient matrix dhz is singular.
!     = 0 Newton's iteration converges.
!
! dhz     Jacobian matrix for Newton's method
! h       function values for Newton's method

!  .. Local Variables ..

      INTEGER          i, iter
      DOUBLE PRECISION nrm_p0, nrm_p1, nrm_dif
      DOUBLE PRECISION dznrm2

!  .. Executable Code ..

      iter = 0

      CALL zgetp1(ap,p0,h)
      nrm_p0 = dznrm2(n,h)

      DO WHILE(iter .le. gmax_nwtn) 

!        One step of Newton's iteration

         CALL zgetdp1(lda,ap,p0,dhz)
         CALL cqrdc_slv(lda,n,dhz,h,info)
         IF(info .eq. 1) RETURN            ! the coefficint matrix is singular
         DO i = 1, n
            p0(i) = p0(i) - h(i)
         ENDDO
         nrm_dif = dznrm2(n,h)

         call zgetp1(ap,p0,h)
         nrm_p1 = dznrm2(n,h)

         IF(nrm_p1 .le. geps_at1) THEN
            info = 0
            RETURN                                         ! p0 is the solution
         ENDIF

         IF(nrm_p1/nrm_p0 .ge. grsdl_fctr) THEN
            info = 2
            RETURN                                 ! residual ratio is too big
         ENDIF

         nrm_p0  = nrm_p1
         iter    = iter + 1
      ENDDO
 
      info = 2

      END
