      SUBROUTINE zonepth1(lda,n,ilow,ihgh,ap,tpwr,p_old_,p_new,
     &                    ntimes,info,
     &                    p_old,v_old,v_new,dhz,h,p0,cf)

      IMPLICIT none

      include '../global.h'

!  .. Scalar Variables ..

      INTEGER lda, n, ntimes, info

!  .. Array Variables ..

      INTEGER ilow(*), ihgh(*)
      DOUBLE PRECISION tpwr(*)
      COMPLEX*16 ap(*), p_old_(*), p_new(*)

!  .. Local array variables ..

      COMPLEX*16 p_old(*), v_old(*), v_new(*)

      COMPLEX*16 dhz(lda,*), h(*), p0(*), cf(*)

!  .. PURPOSE ..

!       This is to follow one homotopy curve of the nonlinear homotopy.
!       The terms in the homotopy have parameter t with power tpwr(*).

!  .. Input and output ..

! p_old_    The initial point on on the curve at t=0. 
! 
!       during processing:
!
! p_old, p_new  The last two points known on the curves. 
!               p_new is the root at t=1 on exit
! v_old, v_new  The tangent vectors of the curve at p_old, p_new.
! stp_old       The step size to get p_old from the point before it. 
! p_new         The last point known on the curve.
! stp_new       The step size to get p_new from p_old. 
! t_new         The corresponding t for p_new.
! stp_c         The current step size to predict the next point.
! info > 0      Tracing curve failed.
!     =  0      Newton's iteration converges.
!     =  -1     Found the root at t=1.

!  .. Local Variables ..

      INTEGER          i, nstp
      DOUBLE PRECISION t_new, stp_old, stp_new, stp_c
      DOUBLE PRECISION stp_init, stp_max

!  .. External function called

      DOUBLE PRECISION dznrm2

!  .. Executable Code ..

  10  IF( ntimes .GT. gmax_ntimes) then
         info = 2       !curve-tracing still fails after max_ntimes times.
         RETURN
      ENDIF

      DO i = 1, n
         p_old(i) = p_old_(i)
      ENDDO
      stp_init = gstp_init/(10.0d0**(ntimes-1))
      stp_max = gstp_max/(10.0d0**(ntimes-1))

      nstp = 0              ! # of steps. For testing only

      DO i = 1, n
         v_old(i) = 0.0d0
      ENDDO

!     To go out one step

      stp_c = stp_init
      CALL z1ststp1(lda,n,ilow,ihgh,ap,tpwr,p_old,stp_c,
     &              p_new,v_new,info,
     &              dhz,h,cf)

      nstp = nstp + 1

      IF(info .GT. 0) THEN
         IF(debug) THEN
           DO i = 1, n
              WRITE(10,*) '  ', p_new(i)
           ENDDO
           WRITE(10,*) '  Failed at z1ststp1.f with info = ', info
           WRITE(10,*) '  The # of times traces : ', ntimes
           WRITE(10,*) '  The # of steps  is : ',nstp
           WRITE(10,*) '  The current t =  ', t_new
         ENDIF

         ntimes = ntimes + 1
         GOTO 10
      ENDIF

      t_new = stp_c

      stp_old = stp_c
      stp_new = stp_c
      stp_c   = stp_c

!     To go out one more step

 20   CALL z1step1(lda,n,ilow,ihgh,ap,tpwr,p_old,v_old,p_new,v_new,
     &             t_new,stp_new,stp_c,info,
     &             dhz,h,p0,cf)
      nstp = nstp + 1

      IF(info .GT. 0) THEN
         IF(debug) THEN
           DO i = 1, n
              WRITE(10,*) '  ', p_new(i)
           ENDDO
           WRITE(10,*) '  Failed at z1step1.f with info = ', info
           WRITE(10,*) '  The # of times traces : ', ntimes
           WRITE(10,*) '  The # of steps  is : ',nstp
           WRITE(10,*) '  The current t =  ', t_new
           IF (t_new .gt. 0.99) THEN   !testing
              CALL zgetp1(ap,p_new,h)
              WRITE(10,*) ' Norm of P(p_new) is : ', dznrm2(n,h)
           ENDIF
         ENDIF

         ntimes = ntimes + 1
         GOTO 10
      ELSE
         IF (info .EQ. -1) THEN                       ! p_new is a root at t=1
            IF(debug) THEN
              WRITE(10,*) '  The root is located at the level t=1'
              WRITE(10,*) '  The # of times traces : ', ntimes
              WRITE(10,*) '  The # of steps  is : ',nstp
            ENDIF

            RETURN 
         ENDIF
      ENDIF

!     To decide next step size	

      IF(stp_c .ge. stp_new .AND. stp_new .ge. stp_old) THEN
         stp_old = stp_new
         stp_new = stp_c
         stp_c   = 2*stp_c
      ELSE
         stp_old = stp_new
         stp_new = stp_c
         stp_c   = stp_c
      ENDIF

      IF(t_new .gt. 0.999) THEN
         stp_c = MIN(stp_c,stp_max*0.1)
      ELSE
         stp_c = MIN(stp_c,stp_max)
      ENDIF
      GOTO 20

      END
