      SUBROUTINE dgetrt(lda,n,r,u,b,b1,roots,tnorts,norts,v,ic)
 
      IMPLICIT none
 
!  .. Scalar Arguments ..
 
      INTEGER lda, n, norts, tnorts, ic
 
!  .. Array Arguments ..
 
      DOUBLE PRECISION r(lda,*), u(lda,*), v(lda,*), b1(*)
      COMPLEX*16 b(*), roots(lda,*)

!  PURPOSE
!  =======
! (1)    solve y^r = b and get x = y^u
!        r11 r12 r13 r14            y1^r11 = b(1)
! r is       r22 r23 r24   to solve y1^r12 y2^r22 = b(2)
!                r33 r34            y1^r13 y2^r23 y3^r33= b(3)
!                    r44            y1^r14 y2^r24 y3^r34 y4^r44= b(4)
! Then     x(1) = y1^u11 y2^u21 y3^u31 y4^u41
!          x(2) = y1^u12 y2^u22 y3^u32 y4^u42
!          x(3) = y1^u13 y2^u23 y3^u33 y4^u43
!          x(4) = y1^u14 y2^u24 y3^u34 y4^u44
! The solutions are put in matrix roots starting from norts+1 -th column
!
! (2)    Solve r^T y = b1  and v = u^Ty
!        v is the normal vector of the facet.
!
!  r        -- double precision array of dimension n by n (input)
!              integer upper triangular matrix, the i-th column is the 
!              power of the non-constan term of the i-th binormial equation
!  u        -- double precision array of simension n by n (input)
!              integer matrix iwith u*a=r and det(u)=1 or -1
!  b        -- complex*16 array of dimension n (input)(changed)
!              the constant terms of the binormial equations
!  b1       -- double precision array of dimension n (input)(changed)
!              the left hand side of the equations for the normal vector
!  roots    -- complex*16 array of dimension n by ? (output)
!              each column of roots is a root for  one of the binormial 
!              polynomial systems
!  norts    -- integer (output)
!              the total number of roots for this facet
!  normalv  -- double precision array of dimension n (output)
!              the normal vector for the facet
!
!  .. Local Variables ..
!
      INTEGER i, j, nn, nogp, stride, itmp1, itmp2, i1, igp, j1
      DOUBLE PRECISION pi, rr, theta
      COMPLEX*16 rt, ctmp
!
!  .. Executable code ..
!
      pi=3.141592653589789963d0
      norts = 1
      do i = 1, n
         norts = norts*nint(r(i,i))
      enddo
!
!     solve the first equation
!
      nogp = 1
!     number of groups in previous level
!
      stride = norts/nint(r(1,1))
!     the times a component rt  repeats itself
!
      nn = nint(r(1,1))
      rr = abs(b(1))**(1.0d0/nn)
      theta=atan2(dimag(b(1)),dble(b(1)))/nn
!
      itmp1 = 1+tnorts
      itmp2 = stride+tnorts
      do i1 = 1, nint(r(1,1))
         rt = rr*cmplx(cos(theta+i1*(2*pi/nn)),sin(theta+i1*(2*pi/nn)))
         do j = itmp1, itmp2 
            roots(1,j) = rt
         enddo
         itmp1 = itmp1 + stride
         itmp2 = itmp2 + stride
      enddo
!     have listed the first component, each repeats stride times
!
      do i = 2, n
!        for the i-th equation
         nogp = nogp*nint(r(i-1,i-1))
         stride = stride/nint(r(i,i))
!        
         nn = nint(r(i,i))
         itmp1 = 1+tnorts
         itmp2 = stride+tnorts
         do igp = 1, nogp
!           for each group
!           to form the right hand side of the equation first (plug the 
!           all previous components, then move the constant factor to the 
!           right hand side
            ctmp = b(i)
            do j1 = 1 , i-1
               ctmp = ctmp/roots(j1,itmp1)**nint(r(j1,i))
            enddo
!           ctmp is the right hand side of the equation
!
            rr = abs(ctmp)**(1.0d0/nn)
            theta=atan2(dimag(ctmp),dble(ctmp))/nn
!
            do i1 = 1, nint(r(i,i))
               rt = rr*cmplx(cos(theta+i1*(2*pi/nn)),
     &                       sin(theta+i1*(2*pi/nn)))
               do j = itmp1, itmp2 
                  roots(i,j) = rt
               enddo
               itmp1 = itmp1 + stride
               itmp2 = itmp2 + stride
            enddo
!           have listed the i-th component, each repeats stride times
         enddo
      enddo
!     have got all the roots for y^r = b 
!
!     to form x = y^u
      do j = 1+tnorts, norts+tnorts
!        for each root
         do i = 1, n
!           for each component of x(i)
            b(i) = 1.0d0
            do i1 = 1, n
               if(nint(u(i1,i)) .ne. 0) then
                  b(i)=b(i)*roots(i1,j)**nint(u(i1,i))
               endif
            enddo
         enddo
         DO i = 1, n
            roots(i,j) = b(i)
         ENDDO
      enddo
!     have formed the x. 
!
!     to calculate the normal vector for this lower facet
!     first r^Ty=b1
      b1(1) = b1(1)/nint(r(1,1))
      do i = 2, n
         do j = 1, i-1
            b1(i) = b1(i) -nint(r(j,i))*b1(j)
         enddo
         b1(i) = b1(i)/nint(r(i,i))
      enddo
!     have got y(=b1)
!
!     to from v = u^Ty
      do i = 1, n
         v(i,ic) = 0.0d0
         DO j = 1, n
            v(i,ic) = v(i,ic) + nint(u(j,i))*b1(j)
         ENDDO
      enddo
!     have got the normal vector
!
      return
      end
