      SUBROUTINE slvinit_un(lda,n,ia,ilow,ns,irep,ap,lft,ptscmb,ncmb, 
     &                      tnorts,roots,rtlow,rthgh,nrmlv,
     &                      a,u,b1,b,ca)
 
      IMPLICIT none 
 
!  .. Scalar Arguments ..
 
      INTEGER lda, n, ns, ncmb, tnorts
 
!  .. Array Arguments ..
 
      INTEGER ia(lda,*), ilow(*), irep(*), ptscmb(*), 
     &        rtlow(*), rthgh(*)
      DOUBLE PRECISION lft(*), nrmlv(lda,*)
      COMPLEX*16 ap(*), roots(lda,*)

!  .. Local array variables ..

      DOUBLE PRECISION a(lda,*), u(lda,*), b1(*)
      COMPLEX*16       b(*), ca(lda,*)

!  ia       -- each column of ia(*,2:*) is a point in the supports
!  ap       -- the coefficents of the terms of the polynomial system
!              corresponding to the points in ia
!  lft      -- the lifting values for the corresponding points in ia
!  ptscmb   -- the indices of the points in the lower facets of
!              type (1, ..,1)
!  ncmb     -- the number of lower facets of type (1,...,1)
!  tnorts   -- the mixed volumn of the system (the number of roots of
!              a generic polynomial system with the support)
!  roots    -- the roots of the polynomial systems corresponding to
!              the lower facets. Each column of roots is a root
!  rtlow    -- rtlow(i)-th column of roots is the first root of the
!              system corresponding to the i-th facet
!  rthgh    -- rthgh(i)-th column of roots is the last root of the
!              system corresponding to the i-th facet
!  nrmlv    -- the i-th column of nrmlv is the normal vector of
!              the i-th facet. (the t-direction of the normal is set
!              to be 1 and is not stored) 
 
!  .. Local Variables ..
 
      INTEGER ic, j, norts, nvtx, icnt, neqs, i, i1, i2, is, itmp
 
!  .. Executable code ..
 
! For each lower facet of type (1,...,1), we need to do:
 
      tnorts = 0                           !! total number of roots got so far
!
      nvtx = ns
      DO i = 1, ns
         nvtx = nvtx + irep(i)          !! # of points in a cell
      ENDDO

      DO ic = 1, ncmb
 
! 1st step : Change the initial system into
!
!     x^{a1} = b(1)
!     x^{a2} = b(2)
!     x^{a3} = b(3)
!     x^{a4} = b(4)
!     with a = (a1, a2, a3, a4)
!     We need to solve x^a = b for initial roots
!     We need to solve a^T v = b1 for normal direction
!
         icnt = (ic-1)*nvtx
         is   = ns
         neqs = 0
         DO i = 1, ns
            IF(irep(i) .EQ. 1) THEN
               icnt = icnt + 1
               neqs = neqs + 1
               DO i1 = 1, n
                  a(i1,neqs) = ia(i1,ptscmb(icnt)) - 
     &                         ia(i1,ptscmb(icnt+1))
               ENDDO
               b(neqs)  = -ap(ptscmb(icnt+1))/ap(ptscmb(icnt))
               b1(neqs) = lft(ptscmb(icnt+1)) - lft(ptscmb(icnt))
               icnt = icnt + 1
            ELSE
               DO i1 = 1, irep(i)
                  neqs = neqs + 1
                  DO i2 = 1, n
                     a(i2,neqs) = ia(i2,ptscmb(icnt+i1))          
     &                          - ia(i2,ptscmb(icnt+irep(i)+1))
                  ENDDO
                  b1(neqs)    = lft(ptscmb(icnt+irep(i)+1)) 
     &                          - lft(ptscmb(icnt+i1))
               ENDDO
               DO i1 = 1, irep(i)
                  ca(1,i1) = ap(ptscmb(icnt+i1))
               ENDDO
               b(neqs-irep(i)+1) = - ap(ptscmb(icnt+irep(i)+1))
               DO i1 = 2, irep(i)
                  is = is + 1
                  itmp = ilow(is) - ilow(i)
                  DO j = 1, irep(i)
                     ca(i1,j) = ap(itmp+ptscmb(icnt+j))
                  ENDDO
                  b(neqs-irep(i)+i1) = -ap(itmp+ptscmb(icnt+irep(i)+1))
               ENDDO
               itmp = irep(i)
               CALL stbl_slv(lda,itmp,ca,b(neqs-itmp+1))
               icnt = icnt + irep(i) + 1
            ENDIF
         ENDDO
!
! 2nd step : get a:=r=u*a which is upper triangular
!
         CALL dformu(lda,n,a,u)
!
! third    : solve y^r = b and get x = y^u
!        r11 r12 r13 r14            y1^r11 = b(1)
! r is       r22 r23 r24   to solve y1^r12 y2^r22 = b(2)
!                r33 r34            y1^r13 y2^r23 y3^r33= b(3)
!                    r44            y1^r14 y2^r24 y3^r34 y4^r44= b(4)
! Then     x(1) = y1^u11 y2^u21 y3^u31 y4^u41
!          x(2) = y1^u12 y2^u22 y3^u32 y4^u42
!          x(3) = y1^u13 y2^u23 y3^u33 y4^u43
!          x(4) = y1^u14 y2^u24 y3^u34 y4^u44
!
! forth    : get the normal vector v of the facet by solving
!            r^T y = b1 and v=u^T y
!        dgetrt is to get all x and v
!
         CALL dgetrt(lda,n,a,u,b,b1,roots,tnorts,norts,nrmlv,ic)
!
         tnorts    = tnorts + norts
         rthgh(ic) = tnorts
      ENDDO
!
      rtlow(1) = 1
      DO i = 1, ncmb-1
         rtlow(i+1) = rthgh(i) + 1
      ENDDO
!
      RETURN
      END
