      Program main_gaoli

      implicit none
 
!     For calculating the mixed volume of a polynomial system
!     by the algorithm given in Tangan Gao and T.Y. Li's paper
!     "Mixed Volume Computation for Semi-Mixed Systems".
 
      integer lda, tnop, maxmvl, maxedges
c     parameter (lda=15, tnop=5000, maxmvl=4000000, maxedges=80000)  

!     If your compiler runs out of memory when it compiles or runs mixvol,
!     comment out the following two lines and set your own parameters 
!     on the previous line

      include 'dimension.h'
      include 'mixvol.h'
 
!     lda >= n + 2
!     tnop >= 2 + total number of terms in the polynomial system
!     maxmvl >= the mixed volume of the polynomial system
!     maxedges >= the numbers of subcells waiting to be extended,
!                 set it as large as possible according your computer

      integer ia(lda,tnop), irep(lda), jlow(lda), jhgh(lda),
     &        ith2is(2*lda), ith2na(2*lda), npn(2*lda),
     &        jlowp(lda,2*lda), jhghp(lda,2*lda),
     &        ib(lda), 
     &        ptscmb1(2*maxedges), 
     &        ptscmb(2*lda*maxmvl), 
     &        jlowc(2*lda), 
     &        j12(2*lda), 
     &        jcmbs(2*lda), jpts(0:2*lda), jlnupts(2*lda),
     &        ptsin(tnop,2*lda), nptsin(2*lda), 
     &        pairs(tnop,tnop), lpindx(tnop,2*lda),
     &        ib_s(lda,maxedges), jpter_s(maxedges)
 
      double precision a(lda,2*lda*tnop), lft(tnop), 
     &                 b(2*lda*tnop), x(lda), c(lda), v(lda),
     &                 binv(lda,lda) 
      double precision x_s(lda,maxedges), binv_s(lda,lda*maxedges)

      integer i, iseed, j, n, ns, i1, icnt
      integer ncmb, mvl
      character*25 filename
 
!     function called
 
      double precision dran2
 
! ... Executable code ...
 
      filename = 'for.mixvol'
      open(2,file=filename)

         read(2,*,END=5) n, ns
         read(2,*,END=5) (irep(i),i=1,ns)
         do i = 1, n
            read(2,*,END=5) jlow(i), jhgh(i)
         enddo
         do j = 1, jhgh(n)
            read(2,*,END=5) (ia(i,j), i = 1, n)
         enddo

      close(2)

      goto 10

 5    print*, 'The support data file is wrong'
      stop

c10   print*, 'total # of points in the support : ', jhgh(ns)
 
c     iseed = 894523
 10   OPEN(UNIT=2, FILE='SRC/SOLVER/iseed.dat')
         read(2,*) iseed
      close(2)

      do j = jlow(1), jhgh(ns)
         lft(j) = 1.0d0+3*dran2(iseed)-dran2(iseed)
      enddo

      OPEN(UNIT=2, FILE='SRC/SOLVER/iseed.dat')
         iseed = dran2(iseed)*700000
         write(2,*) iseed
      close(2)

      call dmxcll(lda,tnop,n,ns,ia,irep,jlow,jhgh,lft,
     &            ptscmb,ncmb,
     &            jcmbs,jlowc,
     &            j12,jpts,jlnupts,
     &            ptscmb1,pairs,ptsin,nptsin,lpindx,
     &            ib,a,b,c,binv,x,v,
     &            ith2is,ith2na,npn,
     &            jlowp,jhghp,
     &            ib_s,x_s,binv_s,jpter_s)

      call mxvl(lda,n,ns,ia,irep,jlow,jhgh,lft,ptscmb,ncmb,
     &          mvl,
     &          binv)

      OPEN(UNIT=2, FILE='SRC/SOLVER/mixcells.h')
         WRITE(2,*) '      PARAMETER(maxmvl=', mvl+5, ',maxncmb=',
     &              ncmb+5, ')'
         WRITE(2,20) '! This file is generated by SOLVER/main_mixvol.f'
 20      FORMAT(80a)
      CLOSE(2)
 
      OPEN(UNIT=2, FILE='for.slvinit')

         WRITE(2,*) 0

         WRITE(2,*) n, ns

         WRITE(2,*) (irep(i),i=1,ns)

         DO i = 1, n
            WRITE(2,*) jlow(i), jhgh(i)
         ENDDO

         DO j = jlow(1), jhgh(n)
            WRITE(2,*) (ia(i,j),i=1,n)
         ENDDO

         DO j = jlow(1), jhgh(n)
            WRITE(2,*) lft(j)
         ENDDO

         WRITE(2,*) ncmb
         j = ns
         DO i = 1, ns
            j = j + irep(i)
         enddo

         DO i = 1, ncmb*j
            WRITE(2,*) ptscmb(i)
         ENDDO

      CLOSE(2)

      print*, '# of Mixed Cells : ', ncmb
      print*, 'Mixed volume     : ', mvl

      stop
      end
