      SUBROUTINE alg2_sp(a,maa,naa,nos,noe,idx,b,x,binv,ib,info,
     &                   lda,s)

      IMPLICIT NONE
 
!  .. Scalar Arguments ..
 
      INTEGER lda, maa, naa, info

!  lda >= naa
 
!     naa by maa : active dimension of a in matrix form
 
!  .. Array Arguments ..
 
      INTEGER nos(*), noe(*), idx(*), ib(*)
      DOUBLE PRECISION a(*), b(*), x(*), binv(lda,*)
 
!  .. Purpose ..
  
!     Solve the LP
!         min x(1)
!         aa^Tx \lq b
!     where aa is an (maa x naa) matrix
!            x is an (naa x 1  ) vector
!            b is an (maa x 1  ) vector.
!     But in this program, the sparsity of the matrix aa is
!     exploited. So the matrix aa is stored a in sparse form:
!                                                -----------
! a   : only the nonzero elements of aa are stored in a(*).
! nos : nos(i) is the position in a of the first nonzero element 
!                 in the i-th row of aa.
! noe : noe(i) is the # of nonzero elements in i-th row of aa.
! idx : idx(i) is the column index of a(i) in aa.
 
!  .. Local array ..
 
      DOUBLE PRECISION s(*)         !! s(SIZE(a,1))
 
!  .. Local variables ..
 
      INTEGER          ell, k, iter, i, j
      DOUBLE PRECISION sigj, vmax, bot, top, ratio, smallnum
 
!  .. Executable
 
      smallnum = 1.0d-10
      iter = 0
      info = 0
!
!**** step 1
!
 10     DO i=1,naa
           IF(ib(i) .EQ. 0) GOTO 20
        ENDDO
        GOTO 40
!
 20     DO i = 1, naa
           IF(ib(i) .EQ. 0) THEN
              s(i) = binv(1,i)
           ENDIF
        ENDDO
!
        vmax = -1.0d0
        k = 0
        DO i = 1, naa
           IF(ib(i) .EQ. 0 .AND. ABS(s(i)) .GT. vmax) THEN
              vmax = ABS(s(i))
              k    = i
           ENDIF
        ENDDO
!
        IF(vmax .GT. smallnum) THEN
           IF(s(k) .GE. 0.0d0) THEN
              DO i = 1, naa
                 s(i) = binv(i,k)
              ENDDO
           ELSE
              DO i = 1, naa
                 s(i) = -binv(i,k)
              ENDDO
           ENDIF
           GOTO 100
        ENDIF
!
 40     DO i = 1, naa
           IF(ib(i) .NE. 0) THEN
              s(i) = binv(1,i)
           ENDIF
        ENDDO
!
        vmax = -1.0d150              !! HUGE(1.0_wp)
        DO i = 1, naa
           IF (ib(i) .NE. 0 .AND. s(i) .GT. vmax) THEN
              vmax = s(i)
              k    = i
           ENDIF
        ENDDO
!
        IF(vmax .LT. smallnum) THEN
           info = 1          !out with optimal solution
           RETURN
        ENDIF
        DO i = 1, naa
           s(i) = binv(i,k)
        ENDDO
!
!**** step 2
!

 100    sigj = 1.0d150        !!HUGE(1.0_wp)
        ell  = 0
        DO i = 1, maa
           DO j = 1, naa
              IF(ib(j) .EQ. i) GOTO 110
           ENDDO
           bot = 0.0d0
           DO j = 0, noe(i)-1
              bot = bot + a(nos(i)+j)*s(idx(nos(i)+j))
           ENDDO
           IF(bot .GE. -smallnum) GOTO 110
           top = -b(i)
           DO j = 0, noe(i)-1
              top = top + a(nos(i)+j)*x(idx(nos(i)+j))
           ENDDO
           ratio = top/bot
           IF(ratio .GE. sigj) GOTO 110
           sigj = ratio
           ell  = i
 110    ENDDO
        IF(ell .EQ. 0) THEN
           info = 2                     !unbounded from below
           RETURN
        ENDIF
!
!**** step 3
!
        DO i = 1, naa
           x(i) = x(i) - sigj*s(i)
        ENDDO
        DO i = 1, naa
           s(i) = 0.0d0
        ENDDO
        DO i = 0, noe(ell)-1
           s(idx(nos(ell)+i)) = a(nos(ell)+i)
        ENDDO
!
!       to find the new B^{-1}
!
        top = 0.0d0
        DO i = 1, naa
           top = top + s(i)*binv(i,k)
        ENDDO
        IF(ABS(top) .LT. smallnum) THEN
           WRITE(*,*) 'singular in phi.f. sum = ', top
           STOP
        ENDIF
!
        top = 1.0d0/top
        DO i = 1, naa
           binv(i,k) = top*binv(i,k)
        ENDDO
        DO j = 1, naa
           IF(j .NE. k) THEN
              top = 0.0d0
              DO i = 1, naa
                 top = top + s(i)*binv(i,j)
              ENDDO
              DO i = 1, naa
                 binv(i,j) = binv(i,j) - top*binv(i,k)
              ENDDO
           ENDIF
        ENDDO
!
        ib(k) = ell
        iter  = iter + 1
!
      GOTO 10
!
      END 
